@@ -13,10 +13,17 @@
 
 open Lib
 
+let latest_ocaml4 = "4.14.2"
+let latest_ocaml5 = "5.2.0" (* Add this number to ocamls below when the next version comes out *)
 let ocamls = [
   (* Fully supported versions *)
-   "4.08.1"; "4.09.1"; "4.10.2"; "4.11.2"; "4.12.1"; "4.13.1"; "5.0.0"; "5.1.1"; "4.14.1";
+  "4.08.1"; "4.09.1"; "4.10.2"; "4.11.2"; "4.12.1"; "4.13.1";
+  "5.0.0"; "5.1.1";
+
+  (* The last elements of the list after 4.14 will be used as default versions *)
+  latest_ocaml4; latest_ocaml5;
 ]
+let start_latests_ocaml = (4, 14)
 
 (* Entry point for the workflow. Workflows are specified as continuations where
    each job is passed as a continuation to the [workflow], terminated with
@@ -65,8 +72,6 @@ let end_workflow ~oc:_ ~workflow:_ = ()
 let ocamls =
   List.map (fun v -> Scanf.sscanf v "%u.%u.%u" (fun major minor _ -> ((major, minor), v))) ocamls
 
-let latest_ocaml = List.fold_left (fun _ (v, _) -> v) (0, 0) ocamls
-
 let platform_ocaml_matrix ?(dir=List.drop_while) ~fail_fast start_version =
   (fail_fast,
    [("ocamlv", List.map snd (dir (fun ocaml -> fst ocaml <> start_version) ocamls))],
@@ -296,13 +301,32 @@ let main_build_job ~analyse_job ~cygwin_job ?section runner start_version ~oc ~w
   (* Intentionally fail fast, no need to run all build if there is a
    * problem in a given version; usually it is functions not defined in lower
    * versions of OCaml. *)
-  let (_fail_fast, matrix, _) = platform_ocaml_matrix ~fail_fast:true start_version in
   let (matrix, includes) =
     if platform = Windows then
-      (("host", ["x86_64-pc-cygwin"; "i686-w64-mingw32"; "x86_64-w64-mingw32"; "i686-pc-windows"; "x86_64-pc-windows"]) ::
-       ("build", ["x86_64-pc-cygwin"]) ::
-       matrix, [])
+      let matrix =
+        let ocaml4 = [
+          "x86_64-pc-cygwin";
+          "i686-w64-mingw32";
+          "x86_64-w64-mingw32";
+          "i686-pc-windows";
+          "x86_64-pc-windows"
+        ] in
+        let ocaml5 = [
+          "x86_64-w64-mingw32";
+          (* "x86_64-pc-windows"; 5.3 needed *)
+        ] in
+        let matrix_elem ocamlv hosts =
+          let elem ocaml host =
+            [("host", host); ("build", "x86_64-pc-cygwin"); ("ocamlv", ocaml)]
+          in
+          List.map (elem ocamlv) hosts
+        in
+        matrix_elem latest_ocaml4 ocaml4
+        @ matrix_elem latest_ocaml5 ocaml5
+      in
+      ([], matrix)
     else
+      let (_fail_fast, matrix, _) = platform_ocaml_matrix ~fail_fast:true start_version in
       (matrix, []) in
   let matrix = ((platform <> Windows), matrix, includes) in
   let needs = if platform = Windows then [analyse_job; cygwin_job] else [analyse_job] in
@@ -351,7 +375,7 @@ let main_test_job ~analyse_job ~build_linux_job ~build_windows_job:_ ~build_macO
     | MacOS -> [analyse_job]  (* This isn't gated on build_macOS_job for speed *)
     | Linux -> [analyse_job; build_linux_job]
   in
-  let matrix = platform_ocaml_matrix ~fail_fast:false latest_ocaml in
+  let matrix = platform_ocaml_matrix ~fail_fast:false start_latests_ocaml in
   let host = host_of_platform platform in
   let ocamlv = "${{ matrix.ocamlv }}" in
   job ~oc ~workflow ?section ~runs_on:(Runner [runner]) ~env:[("OPAM_TEST", "1")] ~matrix ~needs ("Test-" ^ name_of_platform platform)
@@ -391,7 +415,7 @@ let solvers_job ~analyse_job ~build_linux_job ~build_windows_job ~build_macOS_jo
   let only_on target = only_on platform target in
   let needs = [analyse_job; (match platform with Linux -> build_linux_job | Windows -> build_windows_job | MacOS -> build_macOS_job)] in
   let env = [("SOLVER", "${{ matrix.solver }}"); ("OPAMBSROOT", "~/.cache/opam.${{ matrix.solver }}.cached")] in
-  let (fail_fast, matrix, _) = platform_ocaml_matrix ~fail_fast:false latest_ocaml in
+  let (fail_fast, matrix, _) = platform_ocaml_matrix ~fail_fast:false start_latests_ocaml in
   let matrix =
     (fail_fast, ("solver", ["z3"; "0install"])::matrix, [])
   in
@@ -413,7 +437,7 @@ let upgrade_job ~analyse_job ~build_linux_job ~build_windows_job ~build_macOS_jo
   let host = host_of_platform platform in
   let only_on target = only_on platform target in
   let needs = [analyse_job; (match platform with Linux -> build_linux_job | Windows -> build_windows_job | MacOS -> build_macOS_job)] in
-  let matrix = platform_ocaml_matrix ~fail_fast:false latest_ocaml in
+  let matrix = platform_ocaml_matrix ~fail_fast:false start_latests_ocaml in
   let ocamlv = "${{ matrix.ocamlv }}" in
   job ~oc ~workflow ?section ~runs_on:(Runner [runner]) ~needs ~matrix ("Upgrade-" ^ name_of_platform platform)
     ++ only_on Linux (run "Install bubblewrap" ["sudo apt install bubblewrap"])
@@ -472,7 +496,7 @@ let main oc : unit =
     (* These should be identical to the values in appveyor.yml *)
     ("OPAM_REPO", "https://github.com/ocaml/opam-repository.git");
     ("OPAM_TEST_REPO_SHA", "dff745994c64d083a6ba3ddc5a9c28ed0ad0f40a");
-    ("OPAM_REPO_SHA", "dff745994c64d083a6ba3ddc5a9c28ed0ad0f40a");
+    ("OPAM_REPO_SHA", "6eee105e52e098e36949a584c053a18bcb9b2f6b");
     ("SOLVER", "");
     (* Cygwin configuration *)
     ("CYGWIN_MIRROR", "http://mirrors.kernel.org/sourceware/cygwin/");
@@ -482,7 +506,7 @@ let main oc : unit =
   ] in
   let keys = [
     ("archives", "archives-1-${{ hashFiles('src_ext/Makefile.dune', 'src_ext/Makefile.sources', 'src_ext/Makefile', '.github/scripts/common/preamble.sh', '.github/scripts/main/preamble.sh', '.github/scripts/main/archives-cache.sh') }}-${{ env.OPAM_REPO_SHA }}");
-    ("ocaml-cache", "${{ hashFiles('src_ext/Makefile.dune', '.github/scripts/main/ocaml-cache.sh', '.github/scripts/main/preamble.sh', '.github/scripts/main/create-ocaml-cache.sh') }}");
+    ("ocaml-cache", "${{ hashFiles('src_ext/Makefile.dune', '.github/scripts/main/ocaml-cache.sh', '.github/scripts/main/preamble.sh') }}");
     ("cygwin", "${{ hashFiles('.github/scripts/cygwin.cmd') }}-${{ env.CYGWIN_EPOCH }}");
     ("opam-bs-cache", "${{ hashFiles('.github/scripts/main/opam-bs-cache.sh', '*.opam', '.github/scripts/main/preamble.sh') }}");
   ] in
@@ -490,8 +514,8 @@ let main oc : unit =
   ++ analyse_job ~keys ~platforms:[Linux]
   @@ fun analyse_job -> cygwin_job ~analyse_job
   @@ fun cygwin_job -> main_build_job ~analyse_job ~cygwin_job ~section:"Build" Linux (4, 08)
-  @@ fun build_linux_job -> main_build_job ~analyse_job ~cygwin_job Windows latest_ocaml
-  @@ fun build_windows_job -> main_build_job ~analyse_job ~cygwin_job MacOS latest_ocaml
+  @@ fun build_linux_job -> main_build_job ~analyse_job ~cygwin_job Windows start_latests_ocaml
+  @@ fun build_windows_job -> main_build_job ~analyse_job ~cygwin_job MacOS start_latests_ocaml
   @@ fun build_macOS_job -> main_test_job ~analyse_job ~build_linux_job ~build_windows_job ~build_macOS_job ~section:"Opam tests" Linux
   @@ fun _ -> main_test_job ~analyse_job ~build_linux_job ~build_windows_job ~build_macOS_job MacOS
   @@ fun _ -> cold_job ~analyse_job ~build_linux_job ~build_windows_job ~build_macOS_job ~section:"Opam cold" Linux
