@@ -237,33 +237,56 @@ let preprocess_dot_install st nv build_dir =
   in
   files, really_process_dot_install, config
 
-let download_package st nv =
-  log "download_package: %a" (slog OpamPackage.to_string) nv;
+let download_shared_source st url nvs =
+  let labelise pkg_str = OpamStd.List.concat_map ", " pkg_str nvs in
+  log "download_package: %a%a"
+    (slog (fun _ -> labelise OpamPackage.to_string)) ()
+    (slog (fun url -> match url, nvs with
+         | None, _ | _, [_] -> ""
+         | Some url, _ -> " " ^ OpamUrl.to_string (OpamFile.URL.url url))) url;
   if OpamStateConfig.(!r.dryrun) || OpamClientConfig.(!r.fake)
-  then Done None
+  then Done None else
+  let nvs =
+    (* filter out version-pinned packages since we already have their source *)
+    List.filter (fun nv ->
+        let dir = OpamSwitchState.source_dir st nv in
+        not (OpamPackage.Set.mem nv st.pinned &&
+             OpamFilename.exists_dir dir &&
+             OpamStd.Option.Op.(
+               OpamPinned.find_opam_file_in_source nv.name dir >>|
+               (fun (f, lock) ->
+               OpamFile.OPAM.(safe_read f |> with_locked_opt lock)) >>=
+               OpamFile.OPAM.version_opt) = Some nv.version))
+      nvs
+  in
+  if nvs = [] then Done None
   else
-  let dir = OpamSwitchState.source_dir st nv in
-  if OpamPackage.Set.mem nv st.pinned &&
-     OpamFilename.exists_dir dir &&
-     OpamStd.Option.Op.(
-       OpamPinned.find_opam_file_in_source nv.name dir >>|
-       OpamFile.OPAM.safe_read >>=
-       OpamFile.OPAM.version_opt)
-     = Some nv.version
-  then Done None
-  else
-  let print_action msg =
-    OpamConsole.msg "%s retrieved %s.%s  (%s)\n"
+  let print_action =
+    OpamConsole.msg "%s retrieved %s  (%s)\n"
       (if not (OpamConsole.utf8 ()) then "->"
        else OpamActionGraph.
-              (action_color (`Fetch ()) (action_strings (`Fetch ()))))
-      (OpamConsole.colorise `bold (OpamPackage.name_to_string nv))
-      (OpamPackage.version_to_string nv)
-      msg;
+              (action_color (`Fetch []) (action_strings (`Fetch []))))
   in
-  OpamUpdate.cleanup_source st
-    (OpamPackage.Map.find_opt nv st.installed_opams)
-    (OpamSwitchState.opam st nv);
+  let print_single_actions msgs =
+    List.iter (fun (nv, msg) ->
+        print_action
+          (OpamConsole.colorise `bold (OpamPackage.name_to_string nv)
+           ^ "." ^ (OpamPackage.version_to_string nv))
+          msg)
+      msgs
+  in
+  let print_full_action msg =
+    print_action
+      (labelise @@ fun nv ->
+       (OpamConsole.colorise `bold (OpamPackage.name_to_string nv))
+       ^ "." ^ (OpamPackage.version_to_string nv))
+      msg
+  in
+  List.iter (fun nv ->
+      OpamUpdate.cleanup_source st
+        (OpamPackage.Map.find_opt nv st.installed_opams)
+        (OpamSwitchState.opam st nv))
+    nvs;
   OpamProcess.Job.catch (fun e ->
       let na =
         match e with
@@ -272,31 +295,49 @@ let download_package st nv =
       in
       Done (Some na))
   @@ fun () ->
-  OpamUpdate.download_package_source st nv dir @@| function
+  OpamUpdate.download_shared_package_source st url nvs @@| function
   | Some (Not_available (s, l)), _ ->
-    let msg = match s with None -> l | Some s -> s in
-    OpamConsole.error "Failed to get sources of %s: %s"
-      (OpamPackage.to_string nv) msg;
+    let msg = OpamStd.Option.default l s in
+    OpamConsole.error "Failed to get sources of %s%s: %s"
+      (labelise OpamPackage.to_string)
+      (match url, nvs with
+       | None, _ | _, [_] -> ""
+       | Some url, _ ->
+         Printf.sprintf " (%s)" (OpamUrl.to_string (OpamFile.URL.url url)))
+      msg;
     Some (s, l)
-  | _, ((name, Not_available (s, l)) :: _) ->
+  | _, ((nv, name, Not_available (s, l)) :: _) ->
     let msg = match s with None -> l | Some s -> s in
     OpamConsole.error "Failed to get extra source \"%s\" of %s: %s"
       name (OpamPackage.to_string nv) msg;
     Some (s, l)
   | Some (Result msg), _ ->
-    print_action msg; None
+    print_full_action msg; None
   | Some (Up_to_date msg), _ ->
-    print_action msg; None
+    print_full_action msg; None
   | None, [] -> None
   | None, (e :: es as extras) ->
-    if List.for_all (function _, Up_to_date _ -> true | _ -> false) extras then
-      print_action "cached"
-    else begin match e, es with
-      | (_, Result msg), [] -> print_action msg
-      | _, _ -> print_action (Printf.sprintf "%d extra sources" (List.length extras))
-    end;
+    if List.for_all (function _, _, Up_to_date _ -> true | _ -> false) extras then
+      print_full_action "cached"
+    else
+      (match e, es with
+       | (_, _, Result msg), [] -> print_full_action msg
+       | _, _ ->
+         print_single_actions
+           (List.map (fun (nv, _, _) ->
+                nv,
+                (Printf.sprintf "%d extra sources"
+                   (List.length
+                      (List.filter (fun (nv',_,_) ->
+                           OpamPackage.compare nv nv' = 0)
+                          extras))))
+               extras));
     None
 
+let download_package st nv =
+  download_shared_source st
+    (OpamFile.OPAM.url (OpamSwitchState.opam st nv)) [nv]
+
 (* Prepare the package build:
    * apply the patches
    * substitute the files *)
@@ -343,7 +384,8 @@ let prepare_package_build env opam nv dir =
     aux patches
   in
   let subst_patches, subst_others =
-    List.partition (fun f -> List.mem_assoc f patches)
+    List.partition (fun f ->
+        OpamStd.List.mem_assoc OpamFilename.Base.equal f patches)
       (OpamFile.OPAM.substs opam)
   in
   if OpamStateConfig.(!r.dryrun) || OpamClientConfig.(!r.fake) then
@@ -478,25 +520,66 @@ let prepare_package_source st nv dir =
       prepare_package_build (OpamPackageVar.resolve ~opam st) opam nv dir
 
 let compilation_env t opam =
-  let open OpamParserTypes in
   let build_env =
-    List.map (OpamEnv.env_expansion ~opam t) (OpamFile.OPAM.build_env opam)
+    List.map
+      (fun env ->
+         OpamEnv.resolve_separator_and_format
+           (OpamEnv.env_expansion ~opam t env))
+      (OpamFile.OPAM.build_env opam)
+  in
+  let cygwin_env =
+    match OpamCoreConfig.(!r.cygbin) with
+    | Some cygbin ->
+      [ OpamTypesBase.env_update_resolved "PATH" Cygwin cygbin
+          ~comment:"Cygwin path"
+      ] @ (match OpamCoreConfig.(!r.git_location) with
+          | None -> []
+          | Some git_location ->
+            if String.equal cygbin git_location then [] else
+              [ OpamTypesBase.env_update_resolved "PATH" PlusEq
+                  git_location ~comment:"Git binary path"])
+    | None -> []
+  in
+  let shell_sanitization = "shell env sanitization" in
+  let build_env_def = "build environment definition" in
+  let cdpath =
+    OpamTypesBase.env_update_resolved "CDPATH" Eq ""
+      ~comment:shell_sanitization
+  in
+  let makeflags =
+    OpamTypesBase.env_update_resolved "MAKEFLAGS" Eq ""
+      ~comment:shell_sanitization
+  in
+  let makelevel =
+    OpamTypesBase.env_update_resolved "MAKELEVEL" Eq ""
+      ~comment:"make env sanitization"
+  in
+  let pkg_name =
+    OpamTypesBase.env_update_resolved "OPAM_PACKAGE_NAME" Eq
+      (OpamPackage.Name.to_string (OpamFile.OPAM.name opam))
+      ~comment:build_env_def
+  in
+  let pkg_version =
+    OpamTypesBase.env_update_resolved "OPAM_PACKAGE_VERSION" Eq
+      (OpamPackage.Version.to_string (OpamFile.OPAM.version opam))
+      ~comment:build_env_def
+  in
+  let cli =
+    OpamTypesBase.env_update_resolved "OPAMCLI" Eq "2.0"
+      ~comment:"opam CLI version"
   in
   let scrub = OpamClientConfig.(!r.scrubbed_environment_variables) in
   OpamEnv.get_full ~scrub ~set_opamroot:true ~set_opamswitch:true
     ~force_path:true t ~updates:([
-      "CDPATH", Eq, "", Some "shell env sanitization";
-      "MAKEFLAGS", Eq, "", Some "make env sanitization";
-      "MAKELEVEL", Eq, "", Some "make env sanitization";
-      "OPAM_PACKAGE_NAME", Eq,
-      OpamPackage.Name.to_string (OpamFile.OPAM.name opam),
-      Some "build environment definition";
-      "OPAM_PACKAGE_VERSION", Eq,
-      OpamPackage.Version.to_string (OpamFile.OPAM.version opam),
-      Some "build environment definition";
-      "OPAMCLI", Eq, "2.0", Some "opam CLI version";
-    ] @
-      build_env)
+        cdpath;
+        makeflags;
+        makelevel;
+        pkg_name;
+        pkg_version;
+        cli
+      ] @
+        build_env
+        @ cygwin_env)
 
 let installed_opam_opt st nv =
   OpamStd.Option.Op.(
@@ -564,10 +647,13 @@ let make_command st opam ?dir ?text_command (cmd, args) =
     let open OpamStd.Option.Op in
     String.concat " | " [
       OpamVersion.(to_string current);
-      (OpamSysPoll.os () +! "unknown") ^"/"^ (OpamSysPoll.arch () +! "unknown");
+      (let env = st.switch_global.global_variables in
+       Printf.sprintf "%s/%s"
+         (OpamSysPoll.os env +! "unknown")
+         (OpamSysPoll.arch env +! "unknown"));
       (OpamStd.List.concat_map " " OpamPackage.to_string
          OpamPackage.Set.(elements @@
-                          inter st.compiler_packages st.installed_roots));
+                          OpamSwitchState.invariant_root_packages st));
       if OpamPackage.Set.mem nv st.pinned then
         match OpamFile.OPAM.url opam with
         | None -> "pinned"
@@ -578,10 +664,8 @@ let make_command st opam ?dir ?text_command (cmd, args) =
               nv.name
           in
           let rev = OpamProcess.Job.run (OpamRepository.revision src u) in
-          Printf.sprintf "pinned(%s%s%s)"
-            (OpamUrl.to_string u)
-            (OpamStd.Option.to_string (fun s -> "("^s^")")
-               (OpamFile.URL.subpath url))
+          Printf.sprintf "pinned(%s%s)"
+            (OpamUrl.to_string_w_subpath (OpamFile.URL.subpath url) u)
             (OpamStd.Option.to_string
                (fun r -> "#"^OpamPackage.Version.to_string r) rev)
       else
@@ -604,7 +688,9 @@ let make_command st opam ?dir ?text_command (cmd, args) =
   OpamSystem.make_command ~env ~name ?dir ~text
     ~resolve_path:OpamStateConfig.(not !r.dryrun)
     ~metadata:["context", context]
-    ~verbose:(OpamConsole.verbose ())
+    ~verbose:(OpamConsole.verbose () ||
+              OpamPackage.Name.Set.mem (OpamPackage.name nv)
+                OpamClientConfig.(!r.verbose_on))
     cmd args
 
 let remove_commands t nv =
@@ -833,17 +919,18 @@ let remove_package t ?silent ?changes ?force ?build_dir nv =
   else
     remove_package_aux t ?silent ?changes ?force ?build_dir nv
 
-let local_vars ~test ~doc =
+let local_vars ~test ~doc ~dev_setup =
   OpamVariable.Map.of_list [
     OpamVariable.of_string "with-test", Some (B test);
     OpamVariable.of_string "with-doc", Some (B doc);
+    OpamVariable.of_string "with-dev-setup", Some (B dev_setup);
   ]
 
-let build_package t ?(test=false) ?(doc=false) build_dir nv =
+let build_package t ?(test=false) ?(doc=false) ?(dev_setup=false) build_dir nv =
   let opam = OpamSwitchState.opam t nv in
   let commands =
     OpamFilter.commands
-      (OpamPackageVar.resolve ~opam ~local:(local_vars ~test ~doc) t)
+      (OpamPackageVar.resolve ~opam ~local:(local_vars ~test ~doc ~dev_setup) t)
       (OpamFile.OPAM.build opam) @
     (if test then
        OpamFilter.commands (OpamPackageVar.resolve ~opam t)
@@ -892,7 +979,10 @@ let build_package t ?(test=false) ?(doc=false) build_dir nv =
       (OpamPackage.to_string nv) (OpamProcess.string_of_command cmd);
     Done (Some (OpamSystem.Process_error result))
   | None, None ->
-    if commands <> [] && OpamConsole.verbose () then
+    if commands <> [] && (OpamConsole.verbose () ||
+                          OpamPackage.Name.Set.mem (OpamPackage.name nv)
+                            OpamClientConfig.(!r.verbose_on))
+    then
       OpamConsole.msg "%s compiled  %s.%s\n"
         (if not (OpamConsole.utf8 ()) then "->"
          else OpamActionGraph.
@@ -903,12 +993,14 @@ let build_package t ?(test=false) ?(doc=false) build_dir nv =
 
 (* Assumes the package has already been compiled in its build dir.
    Does not register the installation in the metadata! *)
-let install_package t ?(test=false) ?(doc=false) ?build_dir nv =
+let install_package t ?(test=false) ?(doc=false) ?(dev_setup=false) ?build_dir
+    nv =
   let opam = OpamSwitchState.opam t nv in
   let commands =
     OpamFile.OPAM.install opam |>
     OpamFilter.commands
-      (OpamPackageVar.resolve ~opam ~local:(local_vars ~test ~doc) t) |>
+      (OpamPackageVar.resolve ~opam
+         ~local:(local_vars ~test ~doc ~dev_setup) t) |>
     OpamStd.List.filter_map
       (function [] -> None | cmd::args -> Some (cmd, args))
   in
@@ -1028,7 +1120,11 @@ let install_package t ?(test=false) ?(doc=false) ?build_dir nv =
     Done (Right e)
   | Left config, changes ->
     let changes_f = OpamPath.Switch.changes root t.switch nv.name in
-    OpamFile.Changes.write changes_f changes;
+    if OpamStateConfig.(not !r.dryrun) then
+      (log "changes recorded for %s: %a"
+         (OpamPackage.to_string nv)
+         (slog OpamDirTrack.to_summary_string) changes;
+       OpamFile.Changes.write changes_f changes);
     OpamConsole.msg "%s installed %s.%s\n"
       (if not (OpamConsole.utf8 ()) then "->"
        else OpamActionGraph.
