@@ -10,8 +10,6 @@
 
 open OpamTypes
 open OpamPackage.Set.Op
-[@@@ocaml.warning "-33"]
-open OpamDoseCompat
 
 let env ~with_test ~with_doc ~dev nv v =
   match OpamVariable.Full.scope v,
@@ -38,7 +36,7 @@ let get_universe ~with_test ~with_doc ~dev opams =
     u_packages = packages;
     u_action = Query;
     u_installed = OpamPackage.Set.empty;
-    u_available = packages;
+    u_available = lazy packages;
     u_depends =
       OpamPackage.Map.mapi
         (fun nv o ->
@@ -59,7 +57,6 @@ let get_universe ~with_test ~with_doc ~dev opams =
         opams;
     u_installed_roots = OpamPackage.Set.empty;
     u_pinned = OpamPackage.Set.empty;
-    u_base = OpamPackage.Set.empty;
     u_invariant = OpamFormula.Empty;
     u_attrs = [];
     u_reinstall = OpamPackage.Set.empty;
