@@ -24,6 +24,8 @@ type validity
 val cli2_0: OpamCLIVersion.t
 val cli2_1: OpamCLIVersion.t
 val cli2_2: OpamCLIVersion.t
+val cli2_3: OpamCLIVersion.t
+
 (* [cli_from ?platform ?experimental since] validity flag since [since], and no
    removal version. If [experimental] is true, it is marked as is (warning and
    documentation update). If [?platform] is not its default [`all], flag is
@@ -214,6 +216,7 @@ val lock_suffix: ?section:string -> OpamCLIVersion.Sourced.t -> string Term.t
 
 (** Man section name *)
 val package_selection_section: string
+val order_sensible_selector_section : string
 
 (** Some package selection flags. Default section is âckage seletion one *)
 val post: ?section:string -> OpamCLIVersion.Sourced.t -> bool Term.t
