@@ -10,7 +10,6 @@
 (**************************************************************************)
 
 open Cmdliner
-open OpamTypes
 open OpamStateTypes
 open OpamTypesBase
 open OpamStd.Op
@@ -24,7 +23,7 @@ exception InvalidFlagContent of string * (string * string) option
 exception InvalidNewFlag of OpamCLIVersion.Sourced.t * string * OpamCLIVersion.t
 
 let raise_invalid_cli :
-  (OpamCLIVersion.Sourced.t, string option) OpamCompat.Result.t -> 'a
+  (OpamCLIVersion.Sourced.t, string option) result -> 'a
   = function
     | Ok ocli -> raise (InvalidCLI ocli)
     | Error None -> raise (InvalidFlagContent ("cli", None))
@@ -182,7 +181,7 @@ let check_and_run_external_commands () =
     let yes = if yes then Some (Some true) else None in
     OpamCoreConfig.init ?yes ?confirm_level ();
     OpamFormatConfig.init ();
-    let root_dir = OpamStateConfig.opamroot () in
+    let root_from, root_dir = OpamStateConfig.opamroot_with_provenance () in
     let has_init, root_upgraded =
       match OpamStateConfig.load_defaults ~lock_kind:`Lock_read root_dir with
       | None -> (false, false)
@@ -207,11 +206,11 @@ let check_and_run_external_commands () =
     in
     let env =
       if has_init then
-        let updates =
-          ["PATH", OpamParserTypes.PlusEq,
-           OpamFilename.Dir.to_string plugins_bin, None]
-        in
-        OpamStateConfig.init ~root_dir ();
+        let updates = [
+          env_update_resolved "PATH" PlusEq
+            (OpamFilename.Dir.to_string plugins_bin)
+        ] in
+        OpamStateConfig.init ~root_from ~root_dir ();
         match OpamStateConfig.get_switch_opt () with
         | None -> env_array (OpamEnv.get_pure ~updates ())
         | Some sw ->
@@ -235,7 +234,7 @@ let check_and_run_external_commands () =
         let prefixed_name = OpamPath.plugin_prefix ^ name in
         let candidates =
           OpamPackage.packages_of_names
-            (Lazy.force st.available_packages)
+            st.packages
             (OpamPackage.Name.Set.of_list @@
              (OpamStd.List.filter_map
                 (fun s ->
@@ -248,6 +247,14 @@ let check_and_run_external_commands () =
               OpamFile.OPAM.has_flag Pkgflag_Plugin (OpamSwitchState.opam st nv))
             candidates
         in
+        let plugins =
+          (* NOTE: Delay the availablity check as late as possible for performance reasons *)
+          (* See https://github.com/ocaml/opam/issues/5296 *)
+          if OpamPackage.Set.is_empty plugins then
+            plugins
+          else
+            OpamPackage.Set.inter plugins (Lazy.force st.available_packages)
+        in
         let installed = OpamPackage.Set.inter plugins st.installed in
         if OpamPackage.Set.is_empty candidates then (cli, argv)
         else if not OpamPackage.Set.(is_empty installed) && cmd = None then
@@ -324,6 +331,10 @@ let display_cli_error msg =
 let display_cli_error fmt =
   Format.ksprintf display_cli_error fmt
 
+let flush_all_noerror () =
+  (try flush stderr with _ -> ());
+  (try flush stdout with _ -> ())
+
 let rec main_catch_all f =
   try f () with
   | OpamStd.Sys.Exit 0 -> ()
@@ -355,8 +366,10 @@ let rec main_catch_all f =
          "Update done, please now retry your command.\n";
        exit (OpamStd.Sys.get_exit_code `Aborted))
   | e ->
-    flush stdout;
-    flush stderr;
+    OpamStd.Exn.register_backtrace e;
+    (try Sys.set_signal Sys.sigpipe Sys.Signal_default
+     with Invalid_argument _ -> ());
+    flush_all_noerror ();
     if (OpamConsole.verbose ()) then
       OpamConsole.errmsg "'%s' failed.\n"
         (String.concat " " (Array.to_list Sys.argv));
@@ -371,8 +384,9 @@ let rec main_catch_all f =
       | OpamSystem.Process_error result ->
         OpamConsole.errmsg "%s Command %S failed:\n%s\n"
           (OpamConsole.colorise `red "[ERROR]")
-          (try List.assoc "command" result.OpamProcess.r_info with
-           | Not_found -> "")
+          (try
+             OpamStd.List.assoc String.equal "command" result.OpamProcess.r_info
+           with Not_found -> "")
           (Printexc.to_string e);
         OpamConsole.errmsg "%s" (OpamStd.Exn.pretty_backtrace e);
         OpamStd.Sys.get_exit_code `Internal_error
@@ -438,9 +452,13 @@ let run () =
       OpamCommands.get_cmdliner_parser cli, argv
   in
   let argv = Array.of_list argv1 in
-  match Term.eval_choice ~catch:false ~argv default commands with
-  | `Error _ -> exit (OpamStd.Sys.get_exit_code `Bad_arguments)
-  | _        -> exit (OpamStd.Sys.get_exit_code `Success)
+  (* TODO: Get rid of this whenever https://github.com/dbuenzli/cmdliner/pull/161 is available *)
+  let to_new_cmdliner_api (term, info) = Cmd.v info term in
+  let default, default_info = default in
+  let commands = List.map to_new_cmdliner_api commands in
+  match Cmd.eval_value ~catch:false ~argv (Cmd.group ~default default_info commands) with
+  | Error _ -> exit (OpamStd.Sys.get_exit_code `Bad_arguments)
+  | Ok _    -> exit (OpamStd.Sys.get_exit_code `Success)
 
 let json_out () =
   match OpamClientConfig.(!r.json_out) with
@@ -465,9 +483,13 @@ let json_out () =
         (Printexc.to_string e)
 
 let main () =
+  if Sys.win32 then begin
+    (* Disable the critical error handling dialog *)
+    ignore (OpamStubs.setErrorMode (1 lor OpamStubs.getErrorMode ()));
+    OpamStubs.setConsoleToUTF8 ();
+  end;
   OpamStd.Sys.at_exit (fun () ->
-      flush stderr;
-      flush stdout;
+      flush_all_noerror ();
       if OpamClientConfig.(!r.print_stats) then (
         OpamFile.Stats.print ();
         OpamSystem.print_stats ();
