@@ -32,16 +32,12 @@ let default_invariant =
      OpamFormula.Atom
        (`Geq, OpamPackage.Version.of_string "4.05.0"))
 
-let eval_variables = [
+let sys_ocaml_version =
   OpamVariable.of_string "sys-ocaml-version", ["ocamlc"; "-vnum"],
-  "OCaml version present on your system independently of opam, if any";
-  OpamVariable.of_string "sys-ocaml-arch", ["sh"; "-c"; "ocamlc -config 2>/dev/null | tr -d '\\r' | grep '^architecture: ' | sed -e 's/.*: //' -e 's/i386/i686/' -e 's/amd64/x86_64/'"],
-  "Target architecture of the OCaml compiler present on your system";
-  OpamVariable.of_string "sys-ocaml-cc", ["sh"; "-c"; "ocamlc -config 2>/dev/null | tr -d '\\r' | grep '^ccomp_type: ' | sed -e 's/.*: //'"],
-  "Host C Compiler type of the OCaml compiler present on your system";
-  OpamVariable.of_string "sys-ocaml-libc", ["sh"; "-c"; "ocamlc -config 2>/dev/null | tr -d '\\r' | grep '^os_type: ' | sed -e 's/.*: //' -e 's/Win32/msvc/' -e '/^msvc$/!s/.*/libc/'"],
-  "Host C Runtime Library type of the OCaml compiler present on your system";
-]
+  "OCaml version present on your system independently of opam, if any"
+
+let eval_variables =
+  sys_ocaml_version :: OpamEnv.sys_ocaml_eval_variables
 
 let os_filter os =
   FOp (FIdent ([], OpamVariable.of_string "os", None), `Eq, FString os)
@@ -55,6 +51,8 @@ let dragonflybsd_filter = os_filter "dragonfly"
 let not_open_free_bsd_filter =
   FNot (FOr (openbsd_filter,  freebsd_filter))
 let win32_filter = os_filter "win32"
+let not_win32_filter =
+  FOp (FIdent ([], OpamVariable.of_string "os", None), `Neq, FString "win32")
 let sandbox_filter = FOr (linux_filter, macos_filter)
 
 let gpatch_filter =
@@ -107,12 +105,12 @@ let req_dl_tools () =
   in
   let open OpamStd.Option.Op in
   let cmd =
-    (OpamRepositoryConfig.E.fetch ()
+    (OpamRepositoryConfig.E.fetch_t ()
      >>= fun s ->
      match OpamStd.String.split s ' ' with
      | c::_ -> Some c
      | _ -> None)
-    >>+ fun () -> OpamRepositoryConfig.E.curl ()
+    >>+ fun () -> OpamRepositoryConfig.E.curl_t ()
   in
   match cmd with
   | Some cmd ->
@@ -121,15 +119,15 @@ let req_dl_tools () =
 
 let dl_tool () =
   let open OpamStd.Option.Op in
-  (OpamRepositoryConfig.E.fetch ()
-   >>+ fun () -> OpamRepositoryConfig.E.curl ())
+  (OpamRepositoryConfig.E.fetch_t ()
+   >>+ fun () -> OpamRepositoryConfig.E.curl_t ())
   >>| fun cmd -> [(CString cmd), None]
 
 let recommended_tools () =
   let make = OpamStateConfig.(Lazy.force !r.makecmd) in
   [
     [make], None, None;
-    ["cc"], None, None;
+    ["cc"], None, Some not_win32_filter;
   ]
 
 let required_tools ~sandboxing () =
@@ -148,6 +146,16 @@ let required_tools ~sandboxing () =
     ["sandbox-exec"], None, Some macos_filter;
   ] else []
 
+let required_packages_for_cygwin =
+  [
+    "diffutils";
+    "make";
+    "patch";
+    "tar";
+    "unzip";
+    "rsync";
+  ] |> List.map OpamSysPkg.of_string
+
 let init_scripts () = [
   ("sandbox.sh", OpamScript.bwrap), Some bwrap_filter;
   ("sandbox.sh", OpamScript.sandbox_exec), Some macos_filter;
