@@ -64,6 +64,7 @@ type selector =
   | Available
   | Installable
   | Pinned
+  | Latests_only
   | Depends_on of dependency_toggles * atom list
   | Required_by of dependency_toggles * atom list
   | Conflicts_with of package list
@@ -72,6 +73,7 @@ type selector =
   | Pattern of pattern_selector * string
   | Atoms of atom list
   | Flag of package_flag
+  | NotFlag of package_flag
   | Tag of string
   | From_repository of repository_name list
   | Owns_file of filename
@@ -86,6 +88,7 @@ let string_of_selector =
   | Available -> "available" % `cyan
   | Installable -> "installable" % `cyan
   | Pinned -> "pinned" % `cyan
+  | Latests_only -> "latest-only" % `cyan
   | Depends_on (tog,atoms) ->
     Printf.sprintf "%s(%s)"
       ((if tog.recursive then "rec-depends-on" else "depends-on") % `blue)
@@ -125,6 +128,9 @@ let string_of_selector =
   | Flag fl ->
     Printf.sprintf "%s(%s)" ("has-flag" % `green)
       (OpamTypesBase.string_of_pkg_flag fl % `bold)
+  | NotFlag fl ->
+    Printf.sprintf "%s(%s)" ("not-has-flag" % `green)
+      (OpamTypesBase.string_of_pkg_flag fl % `bold)
   | Tag t ->
     Printf.sprintf "%s(%s)" ("has-tag" % `green) (t % `bold)
   | From_repository r ->
@@ -211,6 +217,30 @@ let pattern_selector patterns =
               Atom (Pattern (version_patt, version))])
         patterns)
 
+let uses_depexts = function
+  | Any
+  | Installed
+  | Root
+  | Compiler
+  | Pinned
+  | Latests_only
+  | Pattern _
+  | Atoms _
+  | Flag _
+  | NotFlag _
+  | Tag _
+  | From_repository _
+  | Owns_file _
+    -> false
+  | Available
+  | Installable
+  | Depends_on _
+  | Required_by _
+  | Conflicts_with _
+  | Coinstallable_with _
+  | Solution _
+    -> true
+
 let apply_selector ~base st = function
   | Any -> base
   | Installed -> st.installed
@@ -222,6 +252,12 @@ let apply_selector ~base st = function
       (OpamSwitchState.universe st ~requested:OpamPackage.Set.empty Query)
       base
   | Pinned -> OpamPinned.packages st
+  | Latests_only ->
+    OpamPackage.Name.Map.fold (fun n vset base ->
+        OpamPackage.Set.add
+          (OpamPackage.create n (OpamPackage.Version.Set.max_elt vset))
+          base)
+      (OpamPackage.to_map base) OpamPackage.Set.empty
   | (Required_by ({recursive=true; _} as tog, atoms)
     | Depends_on ({recursive=true; _} as tog, atoms)) as direction ->
     let deps_fun = match direction with
@@ -306,6 +342,10 @@ let apply_selector ~base st = function
     OpamPackage.Set.filter (fun nv ->
         get_opam st nv |> OpamFile.OPAM.has_flag f)
       base
+  | NotFlag f ->
+    OpamPackage.Set.filter (fun nv ->
+        not (get_opam st nv |> OpamFile.OPAM.has_flag f))
+      base
   | Tag t ->
     OpamPackage.Set.filter (fun nv ->
         get_opam st nv |> List.mem t @* OpamFile.OPAM.tags)
