@@ -36,10 +36,8 @@ let string_of_pinned ?(subpath_prefix=true) opam =
 let read_opam_file_for_pinning ?locked ?(quiet=false) name f url =
   let opam0 =
     let dir = OpamFilename.dirname (OpamFile.filename f) in
-    (* don't add aux files for [project/opam] *)
-    let add_files = OpamUrl.local_dir url = Some dir in
     let opam =
-      (OpamFormatUpgrade.opam_file_with_aux ~quiet ~dir ~files:add_files
+      (OpamFormatUpgrade.opam_file_with_aux ~quiet ~dir ~files:false
          ~filename:f) (OpamFile.OPAM.safe_read f)
     in
     if opam = OpamFile.OPAM.empty then None else Some opam
@@ -77,18 +75,15 @@ let get_source_definition ?version ?subpath ?locked st nv url =
     opam
   in
   let open OpamProcess.Job.Op in
-  let url =
-    let u = OpamFile.URL.url url in
-    match OpamUrl.local_dir u, u.OpamUrl.backend with
-    | Some dir, #OpamUrl.version_control ->
-      OpamFile.URL.with_url
-        (OpamUrl.of_string (OpamFilename.Dir.to_string dir))
-        url
-    | _, _ -> url
-  in
   OpamUpdate.fetch_dev_package url srcdir ?subpath nv @@| function
   | Not_available (_,s) -> raise (Fetch_Fail s)
   | Up_to_date _ | Result _ ->
+    let srcdir =
+      let u = OpamFile.URL.url url in
+      match OpamUrl.local_dir u, u.OpamUrl.backend with
+      | Some dir, #OpamUrl.version_control -> dir
+      | _, _ -> srcdir
+    in
     let srcdir = OpamFilename.SubPath.(srcdir /? subpath) in
     match OpamPinned.find_opam_file_in_source ?locked nv.name srcdir with
     | None -> None
