@@ -25,7 +25,6 @@ module E : sig
     | PRECISETRACKING of bool option
     | SAFE of bool option
     | STATUSLINE of OpamStd.Config.when_ option
-    | USEOPENSSL of bool option
     | UTF8 of OpamStd.Config.when_ext option
     | UTF8MSGS of bool option
     | VERBOSE of OpamStd.Config.level option
@@ -69,12 +68,14 @@ type t = private {
   (** The number of log lines displayed on process error. 0 for all *)
   merged_output : bool;
   (** If set, stderr of commands is merged into their stdout *)
-  use_openssl : bool;
-  (** If false, will use built-in hash functions without checking for an openssl
-      executable first *)
   precise_tracking : bool;
   (** If set, will take full md5 of all files when checking diffs (to track
       installations), rather than rely on just file size and mtime *)
+  cygbin: string option;
+  (** Windows specific: the path of binary directory (bin/) of currently used
+      Cygwin install: internal or external Cygwin, or MSYS2. *)
+  git_location: string option;
+  (** Windows specific: the full path of the git binary to use on Windows. *)
   set : bool;
   (** Options have not yet been initialised (i.e. defaults are active) *)
 }
@@ -93,8 +94,9 @@ type 'a options_fun =
   ?keep_log_dir:bool ->
   ?errlog_length:int ->
   ?merged_output:bool ->
-  ?use_openssl:bool ->
   ?precise_tracking:bool ->
+  ?cygbin:string ->
+  ?git_location:string ->
   'a
 
 val default : t
@@ -117,8 +119,8 @@ val initk: 'a -> 'a options_fun
 
 (** Automatic answering levels
     * [`ask]: prompt and ask user
-    * [`no]: answer no to all opam questions
-    * [`yes]: answer yes to all opam questions
+    * [`all_no]: answer no to all opam questions
+    * [`all_yes]: answer yes to all opam questions
     * [`unsafe_yes]: answer yes to all opam question and launch system package
                      command wit non interactive options
     If confirm-level is set (from cli or environment variable), its value is
