@@ -1,6 +1,6 @@
 (**************************************************************************)
 (*                                                                        *)
-(*    Copyright 2012-2015 OCamlPro                                        *)
+(*    Copyright 2012-2020 OCamlPro                                        *)
 (*    Copyright 2012 INRIA                                                *)
 (*                                                                        *)
 (*  All rights reserved. This file is distributed under the terms of the  *)
@@ -9,9 +9,22 @@
 (*                                                                        *)
 (**************************************************************************)
 
+let might_escape ~sep path =
+  let sep =
+    match sep with
+    | `Unix -> Re.char '/'
+    | `Windows -> Re.alt Re.[  char '\\'; char '/' ]
+    | `Unspecified -> Re.str Filename.dir_sep
+  in
+  List.exists (String.equal Filename.parent_dir_name)
+    Re.(split (compile sep) path)
+
 module Base = struct
   include OpamStd.AbstractString
 
+  let compare = String.compare
+  let equal = String.equal
+
   let check_suffix filename s =
     Filename.check_suffix filename s
 
@@ -26,6 +39,9 @@ module Dir = struct
 
   include OpamStd.AbstractString
 
+  let compare = String.compare
+  let equal = String.equal
+
   let of_string dirname =
     let dirname =
       if dirname = "~" then OpamStd.Sys.home ()
@@ -36,7 +52,7 @@ module Dir = struct
           (OpamStd.String.remove_prefix ~prefix:("~"^Filename.dir_sep) dirname)
       else dirname
     in
-    OpamSystem.real_path dirname
+    OpamSystem.real_path (OpamSystem.forward_to_back dirname)
 
   let to_string dirname = dirname
 
@@ -44,6 +60,9 @@ end
 
 let raw_dir s = s
 
+let mk_tmp_dir () =
+  Dir.of_string @@ OpamSystem.mk_temp_dir ()
+
 let with_tmp_dir fn =
   OpamSystem.with_tmp_dir (fun dir -> fn (Dir.of_string dir))
 
@@ -67,10 +86,15 @@ let cwd () =
 let mkdir dirname =
   OpamSystem.mkdir (Dir.to_string dirname)
 
+let exists_dir dirname =
+  try (Unix.stat (Dir.to_string dirname)).Unix.st_kind = Unix.S_DIR
+  with Unix.Unix_error _ -> false
+
 let cleandir dirname =
-  log "cleandir %a" (slog Dir.to_string) dirname;
-  OpamSystem.remove (Dir.to_string dirname);
-  mkdir dirname
+  if exists_dir dirname then
+    (log "cleandir %a" (slog Dir.to_string) dirname;
+     OpamSystem.remove (Dir.to_string dirname);
+     mkdir dirname)
 
 let rec_dirs d =
   let fs = OpamSystem.rec_dirs (Dir.to_string d) in
@@ -95,12 +119,7 @@ let exec dirname ?env ?name ?metadata ?keep_going cmds =
     (fun () -> OpamSystem.commands ?env ?name ?metadata ?keep_going cmds)
 
 let move_dir ~src ~dst =
-  OpamSystem.command ~verbose:(OpamSystem.verbose_for_base_commands ())
-    [ "mv"; Dir.to_string src; Dir.to_string dst ]
-
-let exists_dir dirname =
-  try (Unix.stat (Dir.to_string dirname)).Unix.st_kind = Unix.S_DIR
-  with Unix.Unix_error _ -> false
+  OpamSystem.mv (Dir.to_string src) (Dir.to_string dst)
 
 let opt_dir dirname =
   if exists_dir dirname then Some dirname else None
@@ -139,8 +158,9 @@ type t = {
 }
 
 let create dirname basename =
-  let b1 = Filename.dirname (Base.to_string basename) in
+  let b1 = OpamSystem.forward_to_back (Filename.dirname (Base.to_string basename)) in
   let b2 = Base.of_string (Filename.basename (Base.to_string basename)) in
+  let dirname = OpamSystem.forward_to_back dirname in
   if basename = b2 then
     { dirname; basename }
   else
@@ -164,6 +184,12 @@ let touch t =
 let chmod t p =
   Unix.chmod (to_string t) p
 
+let written_since file =
+  let last_update =
+    (Unix.stat (to_string file)).Unix.st_mtime
+  in
+  (Unix.time () -. last_update)
+
 let of_string s =
   let dirname = Filename.dirname s in
   let basename = Filename.basename s in
@@ -183,16 +209,56 @@ let open_in filename =
   try open_in (to_string filename)
   with Sys_error _ -> raise (OpamSystem.File_not_found (to_string filename))
 
+let open_in_bin filename =
+  try open_in_bin (to_string filename)
+  with Sys_error _ -> raise (OpamSystem.File_not_found (to_string filename))
+
 let open_out filename =
   try open_out (to_string filename)
   with Sys_error _ -> raise (OpamSystem.File_not_found (to_string filename))
 
+let open_out_bin filename =
+  try open_out_bin (to_string filename)
+  with Sys_error _ -> raise (OpamSystem.File_not_found (to_string filename))
+
 let write filename raw =
   OpamSystem.write (to_string filename) raw
 
 let remove filename =
   OpamSystem.remove_file (to_string filename)
 
+let with_open_out_bin_aux open_out_bin filename f =
+  let v, oc =
+    mkdir (dirname filename);
+    try open_out_bin filename
+    with Sys_error _ -> raise (OpamSystem.File_not_found (to_string filename))
+  in
+  try
+    Unix.lockf (Unix.descr_of_out_channel oc) Unix.F_LOCK 0;
+    f oc;
+    close_out oc;
+    v
+  with e ->
+    OpamStd.Exn.finalise e @@ fun () ->
+    close_out oc; remove filename
+
+let with_open_out_bin [@deprecated] =
+  with_open_out_bin_aux (fun f -> (), open_out_bin f)
+
+let with_open_out_bin_atomic filename f =
+  let open_temp_file filename =
+    let mode = [Open_binary] in
+    let perms = 0o666 in
+    let temp_dir = Dir.to_string (dirname filename) in
+    Filename.open_temp_file ~mode ~perms ~temp_dir "opam-atomic" ".tmp"
+  in
+  let temp_file = with_open_out_bin_aux open_temp_file filename f in
+  try
+    Sys.rename temp_file (to_string filename)
+  with Sys_error _ ->
+    OpamSystem.remove_file temp_file;
+    raise (OpamSystem.File_not_found (to_string filename))
+
 let exists filename =
   try (Unix.stat (to_string filename)).Unix.st_kind = Unix.S_REG
   with Unix.Unix_error _ -> false
@@ -220,19 +286,22 @@ let files d =
   let fs = OpamSystem.files (Dir.to_string d) in
   List.rev_map of_string fs
 
+let files_and_links d =
+  let fs = OpamSystem.files_all_not_dir (Dir.to_string d) in
+  List.rev_map of_string fs
+
 let copy ~src ~dst =
   if src <> dst then OpamSystem.copy_file (to_string src) (to_string dst)
 
 let copy_dir ~src ~dst =
   if src <> dst then OpamSystem.copy_dir (Dir.to_string src) (Dir.to_string dst)
 
-let install ?exec ~src ~dst () =
-  if src <> dst then OpamSystem.install ?exec (to_string src) (to_string dst)
+let install ?warning ?exec ~src ~dst () =
+  if src <> dst then OpamSystem.install ?warning ?exec (to_string src) (to_string dst)
 
 let move ~src ~dst =
   if src <> dst then
-    OpamSystem.command ~verbose:(OpamSystem.verbose_for_base_commands ())
-      [ "mv"; to_string src; to_string dst ]
+    OpamSystem.mv (to_string src) (to_string dst)
 
 let readlink src =
   if exists src then
@@ -310,6 +379,9 @@ let extract_in filename dirname =
 let extract_in_job filename dirname =
   OpamSystem.extract_in_job (to_string filename) ~dir:(Dir.to_string dirname)
 
+let make_tar_gz_job filename dirname =
+  OpamSystem.make_tar_gz_job (to_string filename) ~dir:(Dir.to_string dirname)
+
 type generic_file =
   | D of Dir.t
   | F of t
@@ -365,6 +437,7 @@ let link ?(relative=false) ~target ~link =
       Filename.concat back forward
   in
   OpamSystem.link target (to_string link)
+[@@ocaml.warning "-16"]
 
 let patch ?preprocess filename dirname =
   OpamSystem.patch ?preprocess ~dir:(Dir.to_string dirname) (to_string filename)
@@ -383,14 +456,14 @@ let with_flock flag ?dontblock file f =
           else
             "/dev/null"
         in
-        let ch = Pervasives.open_out null in
+        let ch = Stdlib.open_out null in
         Unix.descr_of_out_channel ch, Some ch
       | fd ->
         fd, None
     in
     let r = f fd in
     OpamSystem.funlock lock;
-    OpamStd.Option.iter Pervasives.close_out ch;
+    OpamStd.Option.iter Stdlib.close_out ch;
     r
   with e ->
     OpamStd.Exn.finalise e @@ fun () ->
@@ -443,6 +516,16 @@ let prettify s =
   prettify_path (to_string s)
 
 let to_json x = `String (to_string x)
+let of_json = function
+  | `String x -> (try Some (of_string x) with _ -> None)
+  | _ -> None
+
+let compare {dirname; basename} f =
+  let dir = Dir.compare dirname f.dirname in
+  if dir <> 0 then dir else
+    Base.compare basename f.basename
+
+let equal f g = compare f g = 0
 
 module O = struct
   type tmp = t
@@ -450,11 +533,33 @@ module O = struct
   let compare = compare
   let to_string = to_string
   let to_json = to_json
+  let of_json = of_json
 end
 
 module Map = OpamStd.Map.Make(O)
 module Set = OpamStd.Set.Make(O)
 
+module SubPath = struct
+
+  include OpamStd.AbstractString
+
+  let compare = String.compare
+  let equal = String.equal
+
+  let of_string s =
+    OpamSystem.back_to_forward s
+    |> OpamStd.String.remove_prefix ~prefix:"./"
+    |> of_string
+  let to_string = OpamSystem.forward_to_back
+  let normalised_string s = s
+
+  let (/) d s = d / to_string s
+  let (/?) d = function
+    | None -> d
+    | Some s -> d / to_string s
+
+end
+
 module Op = struct
 
   let (/) = (/)
@@ -513,12 +618,42 @@ module Attribute = struct
           | None   -> []
           | Some p -> ["perm", `String (string_of_int p)])
 
+  let of_json = function
+    | `O dict ->
+      begin try
+          let open OpamStd.Option.Op in
+          Base.of_json (OpamStd.List.assoc String.equal "base" dict)
+          >>= fun base ->
+          OpamHash.of_json (OpamStd.List.assoc String.equal "md5" dict)
+          >>= fun md5 ->
+          let perm =
+            if not (OpamStd.List.mem_assoc String.equal "perm" dict) then None
+            else match OpamStd.List.assoc String.equal "perm" dict with
+              | `String hash ->
+                (try Some (int_of_string hash) with _ -> raise Not_found)
+              | _ -> raise Not_found
+          in
+          Some { base; md5; perm }
+        with Not_found -> None
+      end
+    | _ -> None
+
+  let compare {base; md5; perm} a =
+    let base = Base.compare base a.base in
+    if base <> 0 then base else
+    let md5 = OpamHash.compare md5 a.md5 in
+    if md5 <> 0 then md5 else
+      OpamStd.Option.compare Int.compare perm a.perm
+
+  let equal a b = compare a b = 0
+
   module O = struct
     type tmp = t
     type t = tmp
     let to_string = to_string
     let compare = compare
     let to_json = to_json
+    let of_json = of_json
   end
 
   module Set = OpamStd.Set.Make(O)
