@@ -12,6 +12,9 @@
 (** Higher level file and directory name manipulation AND file operations,
     wrappers on OpamSystem using the filename type *)
 
+(* Returns [true] if string contains '..' between directory separators *)
+val might_escape: sep:[`Unix | `Windows | `Unspecified ] -> string -> bool
+
 (** Basenames *)
 module Base: sig
   include OpamStd.ABSTRACT
@@ -32,7 +35,8 @@ val cwd: unit -> Dir.t
 (** Remove a directory *)
 val rmdir: Dir.t -> unit
 
-(** Cleans the contents of a directory, but keeps the directory in place. *)
+(** Cleans the contents of a directory, but keeps the directory in place.
+    Noop if directory doesn't exists. *)
 val cleandir: Dir.t -> unit
 
 (** Removes an empty directory, as well as any empty leading path components.
@@ -142,6 +146,17 @@ val open_in_bin: t -> in_channel
 val open_out: t -> out_channel
 val open_out_bin: t -> out_channel
 
+(** [with_open_out_bin filename f] opens [f] and passes the out_channel to [f].
+    If [f] raises an exception, then [filename] is deleted and then exception is
+    propagated. The out_channel does not have to be closed by [f]. *)
+val with_open_out_bin: t -> (out_channel -> unit) -> unit
+[@@deprecated "use with_open_out_bin_atomic instead"]
+
+(** [with_open_out_bin_atomic filename f] opens [f] and passes the out_channel to [f].
+    If [f] raises an exception, then [filename] will be unaltered and then exception is
+    propagated. The out_channel does not have to be closed by [f]. *)
+val with_open_out_bin_atomic: t -> (out_channel -> unit) -> unit
+
 (** Removes everything in [filename] if existed. *)
 val remove: t -> unit
 
@@ -322,5 +337,21 @@ module Attribute: sig
 
 end
 
+(* Subpathes handling *)
+module SubPath: sig
+
+  include OpamStd.ABSTRACT
+
+  (* Directory concatenation *)
+  val (/): Dir.t -> t -> Dir.t
+
+  (* Directory concatenation with an optional argument *)
+  val (/?): Dir.t -> t option -> Dir.t
+
+  (* Subpath string with no file separator conversion *)
+  val normalised_string: t -> string
+
+end
+
 (** Convert a filename to an attribute, relatively to a root *)
 val to_attribute: Dir.t -> t -> Attribute.t
