@@ -219,7 +219,9 @@ module Make (G : G) = struct
         (* Generate the remaining nodes in topological order *)
         let remaining =
           G.Topological.fold (fun n remaining ->
-              if M.mem n results || List.mem_assoc n errors then remaining
+              if M.mem n results
+              || OpamStd.List.mem_assoc G.V.equal n errors then
+                remaining
               else n::remaining)
             g [] in
         raise (Errors (M.keys results, List.rev errors, List.rev remaining))
@@ -239,9 +241,9 @@ module Make (G : G) = struct
              (fun pools ->
                 let slots = get_slots nslots n in
                 OpamStd.List.concat_map " " (fun (pool, jobs) ->
-                  let nslots =
-                    OpamStd.Option.of_Not_found (List.assoc pool) slots
-                  in
+                    let nslots =
+                      OpamStd.List.assoc_opt S.equal pool slots
+                    in
                   Printf.sprintf "%s/%d"
                     (match nslots with
                      | None -> "-"
@@ -275,7 +277,7 @@ module Make (G : G) = struct
           | _ -> OpamProcess.wait_one (List.map fst processes)
         with e -> fail (fst (snd (List.hd processes))) e
       in
-      let n,cont = List.assoc process processes in
+      let n,cont = OpamStd.(List.assoc Compare.equal process processes) in
       log "Collected task for job %a (ret:%d)"
         (slog (string_of_int @* V.hash)) n result.OpamProcess.r_code;
       let next =
@@ -383,8 +385,12 @@ module MakeGraph (X: VERTEX) = struct
       let vertex_to_json (i, v) = (string_of_int i, X.to_json v) in
       `O (List.map vertex_to_json vertex_map) in
     let edges =
-      let vertex_inv_map = List.map (fun (i, v) -> (v, i)) vertex_map in
-      let index v = List.assoc v vertex_inv_map in
+      let module VertexMap = Map.Make(Vertex) in
+      let vertex_inv_map =
+        List.fold_left (fun m (i, v) -> VertexMap.add v i m)
+          VertexMap.empty vertex_map
+      in
+      let index v = VertexMap.find v vertex_inv_map in
       let index_to_json v = `String (string_of_int (index v)) in
       let edge_to_json edge =
         let () = E.label edge in
@@ -404,10 +410,12 @@ module MakeGraph (X: VERTEX) = struct
   let of_json : t OpamJson.decoder = function
     | `O dict ->
       begin try
-          let vertices_json = match List.assoc "vertices" dict with
+          let vertices_json =
+            match OpamStd.List.assoc String.equal "vertices" dict with
             | `O vertices -> vertices
             | _ -> raise Not_found in
-          let edges_json = match List.assoc "edges" dict with
+          let edges_json =
+            match OpamStd.List.assoc String.equal "edges" dict with
             | `A edges -> edges
             | _ -> raise Not_found in
           let vertex_map =
@@ -423,12 +431,14 @@ module MakeGraph (X: VERTEX) = struct
             let int_of_jsonstring = function
               | `String s -> (try int_of_string s with _ -> raise Not_found)
               | _ -> raise Not_found in
-            let find kj = List.assoc (int_of_jsonstring kj) vertex_map in
+            let find kj =
+              OpamStd.List.assoc Int.equal (int_of_jsonstring kj) vertex_map
+            in
             let edge_of_json = function
               | `O dict ->
-                let src = find (List.assoc "src" dict) in
+                let src = find (OpamStd.List.assoc String.equal "src" dict) in
                 let label = () in
-                let dst = find (List.assoc "dst" dict) in
+                let dst = find (OpamStd.List.assoc String.equal "dst" dict) in
                 E.create src label dst
               | _ -> raise Not_found
             in List.map edge_of_json edges_json
