@@ -903,19 +903,23 @@ module OpamSys = struct
       let split = if clean then OpamString.split else OpamString.split_delim in
       split path path_sep
 
-  let with_process_in cmd args f =
+  let process_in cmd args =
     if Sys.win32 then
       assert false;
-    let path = split_path_variable (Env.get "PATH") in
-    let cmd =
-      List.find Sys.file_exists (List.map (fun d -> Filename.concat d cmd) path)
-    in
-    let ic = Unix.open_process_in (cmd^" "^args) in
+    let env = Env.raw_env () in
     try
-      let r = f ic in
-      ignore (Unix.close_process_in ic) ; r
-    with exn ->
-      ignore (Unix.close_process_in ic) ; raise exn
+      let path = split_path_variable (Env.get "PATH") in
+      let cmd =
+        List.find OpamStubs.is_executable
+          (List.map (fun d -> Filename.concat d cmd) path)
+      in
+      let args = Array.of_list (cmd :: args) in
+      let (ic, _, _) as p = Unix.open_process_args_full cmd args env in
+      let r = input_line ic in
+      match Unix.close_process_full p with
+      | Unix.WEXITED 0 -> Some r
+      | WEXITED _ | WSIGNALED _ | WSTOPPED _ -> None
+    with Unix.Unix_error _ | Sys_error _ | End_of_file | Not_found -> None
 
   let tty_out = Unix.isatty Unix.stdout
 
@@ -932,24 +936,16 @@ module OpamSys = struct
   )
 
   let get_terminal_columns () =
-    let fallback = 80 in
-    let cols =
-      try (* terminfo *)
-        with_process_in "tput" "cols"
-          (fun ic -> int_of_string (input_line ic))
-      with
-      | Unix.Unix_error _ | Sys_error _ | Failure _ | End_of_file | Not_found ->
-        try (* GNU stty *)
-          with_process_in "stty" "size"
-            (fun ic ->
-               match OpamString.split (input_line ic) ' ' with
-               | [_ ; v] -> int_of_string v
-               | _ -> failwith "stty")
-        with
-        | Unix.Unix_error _ | Sys_error _ | Failure _
-        | End_of_file | Not_found -> fallback
-    in
-    if cols > 0 then cols else fallback
+    try int_of_string (Env.get "COLUMNS") with
+    | Not_found | Failure _ ->
+      let fallback = 80 in
+      let cols =
+        if tty_out then
+          OpamStubs.get_stdout_ws_col ()
+        else
+          fallback
+      in
+      if cols > 0 then cols else fallback
 
   let win32_get_console_width default_columns =
     try
@@ -995,19 +991,21 @@ module OpamSys = struct
 
   let etc () = "/etc"
 
-  let uname =
+  let memo_command =
     let memo = Hashtbl.create 7 in
-    fun arg ->
-      try Hashtbl.find memo arg with Not_found ->
+    fun cmd arg ->
+      try Hashtbl.find memo (cmd, arg) with Not_found ->
         let r =
-          try
-            with_process_in "uname" arg
-              (fun ic -> Some (OpamString.strip (input_line ic)))
-          with Unix.Unix_error _ | Sys_error _ | Not_found -> None
+          match process_in cmd [arg] with
+          | None -> None
+          | Some x -> Some (OpamString.strip x)
         in
-        Hashtbl.add memo arg r;
+        Hashtbl.add memo (cmd, arg) r;
         r
 
+  let uname = memo_command "uname"
+  let getconf = memo_command "getconf"
+
   let system =
     let system = Lazy.from_fun OpamStubs.getPathToSystem in
     fun () -> Lazy.force system
@@ -1116,12 +1114,11 @@ module OpamSys = struct
         Some (Unix.readlink (Filename.concat dir "exe"))
       with e ->
         fatal e;
-        try
-          with_process_in "ps"
-            (Printf.sprintf "-p %d -o comm= 2>/dev/null" ppid)
-            (fun ic -> Some (input_line ic))
+        match
+          process_in "ps" ["-p"; string_of_int ppid; "-o"; "comm="]
         with
-        | Unix.Unix_error _ | Sys_error _ | Failure _ | End_of_file | Not_found ->
+        | Some _ as x -> x
+        | None ->
             try
               let c = open_in_bin ("/proc/" ^ string_of_int ppid ^ "/cmdline") in
               begin try
@@ -1246,40 +1243,11 @@ module OpamSys = struct
   (* OCaml 4.05.0 no longer follows the updated PATH to resolve commands. This
      makes unqualified commands absolute as a workaround. *)
   let resolve_command =
-    let check_perms =
-      if Sys.win32 then fun f ->
-        try (Unix.stat f).Unix.st_kind = Unix.S_REG
-        with e -> fatal e; false
-      else fun f ->
-        try
-          let {Unix.st_uid; st_gid; st_perm; st_kind; _} = Unix.stat f in
-          if st_kind <> Unix.S_REG then false else
-          let groups =
-            IntSet.of_list (Unix.getegid () :: Array.to_list (Unix.getgroups ()))
-          in
-          let mask =
-            if Unix.geteuid () = (st_uid : int) then
-              0o100
-            else if IntSet.mem st_gid groups then
-              0o010
-            else
-              0o001
-          in
-          if (st_perm land mask) <> 0 then
-            true
-          else
-          match OpamACL.get_acl_executable_info f st_uid with
-          | None -> false
-          | Some [] -> true
-          | Some gids ->
-            not (IntSet.is_empty (IntSet.inter (IntSet.of_list gids) groups))
-        with e -> fatal e; false
-    in
     let resolve ?dir env name =
       if not (Filename.is_relative name) then begin
         (* absolute path *)
         if not (Sys.file_exists name) then `Not_found
-        else if not (check_perms name) then `Denied
+        else if not (OpamStubs.is_executable name) then `Denied
         else `Cmd name
       end else if is_external_cmd name then begin
         (* relative path *)
@@ -1288,7 +1256,7 @@ module OpamSys = struct
           | Some d -> Filename.concat d name
         in
         if not (Sys.file_exists cmd) then `Not_found
-        else if not (check_perms cmd) then `Denied
+        else if not (OpamStubs.is_executable cmd) then `Denied
         else `Cmd cmd
       end else
       (* bare command, lookup in PATH *)
@@ -1302,7 +1270,7 @@ module OpamSys = struct
          expected name but not the right permissions are skipped silently.
          Therefore, only two outcomes are possible in that case, [`Cmd ..] or
          [`Not_found]. *)
-      match List.find check_perms possibles with
+      match List.find OpamStubs.is_executable possibles with
       | cmdname -> `Cmd cmdname
       | exception Not_found ->
         if possibles = [] then
@@ -1854,6 +1822,9 @@ module Config = struct
   let env_string var =
     env (fun s -> s) var
 
+  let env_string_list var =
+    env (fun s -> OpamString.split s ',') var
+
   let env_float var =
     env float_of_string var
 
