@@ -130,6 +130,7 @@ module Config: sig
   val with_switch: switch -> t -> t
   val with_switch_opt: switch option -> t -> t
 
+  (** The list of switches, ordered by most recently used first *)
   val with_installed_switches: switch list -> t -> t
 
   (** Repository updates *)
@@ -168,6 +169,11 @@ module Config: sig
   val with_depext_run_installs: bool -> t -> t
   val with_depext_cannot_install: bool -> t -> t
   val with_depext_bypass: OpamSysPkg.Set.t -> t -> t
+  val with_sys_pkg_manager_cmd: filename OpamStd.String.Map.t -> t -> t
+
+  val with_swh_fallback: bool -> t -> t
+  val with_git_location: dirname -> t -> t
+  val with_git_location_opt: dirname option -> t -> t
 
   (** Return the opam version *)
   val opam_version: t  -> opam_version
@@ -185,6 +191,8 @@ module Config: sig
   (** Return the OCaml switch *)
   val switch: t -> switch option
 
+  val previous_switch : t -> switch option
+
   val installed_switches: t -> switch list
 
   (** Return the number of jobs defined *)
@@ -221,6 +229,14 @@ module Config: sig
   val depext_cannot_install: t -> bool
   val depext_bypass: t -> OpamSysPkg.Set.t
 
+  val sys_pkg_manager_cmd: t -> filename OpamStd.String.Map.t
+
+  (* Enable or disable Software Heritage fallback for unreachable package
+     sources *)
+  val swh_fallback: t -> bool
+
+  val git_location: t -> dirname option
+
   val fields: (string * (t, value) OpamPp.field_parser) list
 
   (** All file fields as print-AST, Fields within sections are
@@ -253,6 +269,8 @@ module InitConfig: sig
   val recommended_tools: t -> (string list * string option * filter option) list
   val required_tools: t -> (string list * string option * filter option) list
   val init_scripts: t -> ((string * string) * filter option) list
+  val sys_pkg_manager_cmd: t -> filename OpamStd.String.Map.t
+  val git_location: t -> dirname option
 
   val with_opam_version: opam_version -> t -> t
   val with_repositories:
@@ -304,7 +322,8 @@ module URL: sig
   include IO_FILE
 
   val create:
-    ?mirrors:url list -> ?checksum:OpamHash.t list -> ?subpath:string ->
+    ?mirrors:url list -> ?checksum:OpamHash.t list ->
+    ?swhid:OpamSWHID.t -> ?subpath:subpath ->
     url -> t
 
   (** URL address *)
@@ -314,14 +333,18 @@ module URL: sig
 
   (** Archive checksum *)
   val checksum: t -> OpamHash.t list
+  val swhid: t -> OpamSWHID.t option
 
   (** Constructor *)
   val with_url: url -> t -> t
   val with_checksum: OpamHash.t list -> t -> t
-  val with_subpath: string -> t -> t
-  val with_subpath_opt: string option -> t -> t
+  val with_mirrors: OpamUrl.t list -> t -> t
+  val with_swhid: OpamSWHID.t -> t -> t
+  val with_swhid_opt: OpamSWHID.t option -> t -> t
+  val with_subpath: subpath -> t -> t
+  val with_subpath_opt: subpath option -> t -> t
 
-  val subpath: t -> string option
+  val subpath: t -> subpath option
 
 end
 
@@ -342,7 +365,7 @@ module OPAM: sig
     conflict_class : name list;
     available  : filter;
     flags      : package_flag list;
-    env        : env_update list;
+    env        : (spf_unresolved, euok_writeable) env_update list;
 
     (* Build instructions *)
     build      : command list;
@@ -353,7 +376,7 @@ module OPAM: sig
     (* Auxiliary data affecting the build *)
     substs     : basename list;
     patches    : (basename * filter option) list;
-    build_env  : env_update list;
+    build_env  : (spf_unresolved, euok_writeable) env_update list;
     features   : (OpamVariable.t * filtered_formula * string) list;
     extra_sources: (basename * URL.t) list;
 
@@ -391,6 +414,9 @@ module OPAM: sig
     (* Names and hashes of the files below files/ *)
     extra_files: (OpamFilename.Base.t * OpamHash.t) list option;
 
+    (* Origin with the extension *)
+    locked: string option;
+
     format_errors: (string * OpamPp.bad_format) list;
 
     (* Deprecated, for compat and proper linting *)
@@ -409,12 +435,16 @@ module OPAM: sig
 
   (** Returns the opam value (including url, descr) with all non-effective (i.e.
       user-directed information that doesn't change opam's view on the package)
-      fields set to their empty values. Useful for comparisons. *)
-  val effective_part: t -> t
+      fields set to their empty values. Useful for comparisons.
+      @param ?modulo_state if [true], eliminates the fields relying on the state
+      of the switch (depends, available, …). This is [false] by default. *)
+  val effective_part: ?modulo_state:bool -> t -> t
 
   (** Returns true if the effective parts of the two package definitions are
-      equal *)
-  val effectively_equal: t -> t -> bool
+      equal.
+      @param ?modulo_state if [true], considers the fields relying on the state
+      of the switch (depends, available, …) equal. This is [false] by default *)
+  val effectively_equal: ?modulo_state:bool -> t -> t -> bool
 
   (** Compares two package definitions, ignoring the virtual fields bound to
       file location ([metadata_dir]...) *)
@@ -447,7 +477,7 @@ module OPAM: sig
   val substs: t -> basename list
 
   (** List of environment variables to set-up for the build *)
-  val build_env: t -> env_update list
+  val build_env: t -> (spf_unresolved, [> euok_writeable]) env_update list
 
   (** List of command to run for building the package *)
   val build: t -> command list
@@ -537,7 +567,7 @@ module OPAM: sig
   val has_flag: package_flag -> t -> bool
 
   (** The environment variables that this package exports *)
-  val env: t -> env_update list
+  val env: t -> (spf_unresolved, [> euok_writeable]) env_update list
 
   val descr: t -> Descr.t option
 
@@ -562,6 +592,9 @@ module OPAM: sig
   (** Names and hashes of the files below files/ *)
   val extra_files: t -> (OpamFilename.Base.t * OpamHash.t) list option
 
+  (** From locked opam file *)
+  val locked: t -> string option
+
   (** Looks up the extra files, and returns their full paths, relative path to
       the package source, and hash. Doesn't check the hashes. *)
   val get_extra_files:
@@ -619,7 +652,7 @@ module OPAM: sig
   (** Construct as [substs] *)
   val with_substs: basename list -> t -> t
 
-  val with_build_env: env_update list -> t -> t
+  val with_build_env: (spf_unresolved, euok_writeable) env_update list -> t -> t
 
   val with_available: filter -> t -> t
 
@@ -647,7 +680,7 @@ module OPAM: sig
 
   val with_tags: string list -> t -> t
 
-  val with_env: env_update list -> t -> t
+  val with_env: (spf_unresolved, euok_writeable) env_update list -> t -> t
 
   val with_dev_repo: url -> t -> t
 
@@ -683,6 +716,8 @@ module OPAM: sig
   val with_extra_files: (OpamFilename.Base.t * OpamHash.t) list -> t -> t
   val with_extra_files_opt: (OpamFilename.Base.t * OpamHash.t) list option -> t -> t
 
+  val with_locked_opt: string option -> t -> t
+
   val with_format_errors: (string * OpamPp.bad_format) list -> t -> t
 
   (** Prints to a string, while keeping the format of the original file as much
@@ -725,6 +760,9 @@ module OPAM: sig
       sections can be accessed through [section.field]. *)
   val print_field_as_syntax: string -> t -> value option
 
+  (** x-field name for path rewriting on windows *)
+  val rewrite_xfield: string
+
 end
 
 (** Compiler aliases: [$opam/aliases]. Deprecated, used only for migration *)
@@ -763,7 +801,15 @@ end
 module PkgList: IO_FILE with type t = package_set
 
 (** Cached environment updates (<switch>/environment) *)
-module Environment: IO_FILE with type t = env_update list
+module Environment : sig
+  include IO_FILE with type t = (spf_resolved, euok_writeable) env_update list
+
+  val read: t typed_file -> (spf_resolved, [> euok_writeable ]) env_update list
+  val read_opt: t typed_file -> (spf_resolved, [> euok_writeable ]) env_update list option
+  val safe_read: t typed_file -> (spf_resolved, [> euok_writeable ]) env_update list
+  val read_from_channel: ?filename:t typed_file -> in_channel -> (spf_resolved, [> euok_writeable ]) env_update list
+  val read_from_string: ?filename:t typed_file -> string -> (spf_resolved, [> euok_writeable ]) env_update list
+end
 
 (** Compiler version [$opam/compilers/]. Deprecated, only used to upgrade old
     data *)
@@ -776,7 +822,7 @@ module Comp: sig
 
   (** Create a pre-installed compiler description file *)
   val create_preinstalled:
-    compiler -> compiler_version -> name list -> env_update list -> t
+    compiler -> compiler_version -> name list -> (spf_unresolved, euok_writeable) env_update list -> t
 
   (** Is it a pre-installed compiler description file *)
   val preinstalled: t -> bool
@@ -811,7 +857,7 @@ module Comp: sig
 
   (** Environment variable to set-up before running commands in the
       subtree *)
-  val env: t -> env_update list
+  val env: t -> (spf_unresolved, euok_writeable) env_update list
 
   val tags: t -> string list
 
@@ -882,6 +928,48 @@ module Dot_install: sig
   (** List of other files to install *)
   val misc: t -> (basename optional * filename) list
 
+  (** List of files to install in $bin/ *)
+  val with_bin : (basename optional * basename option) list -> t -> t
+
+  (** List of files to install in $sbin/ *)
+  val with_sbin : (basename optional * basename option) list -> t -> t
+
+  (** List of files to install in $lib/ *)
+  val with_lib : (basename optional * basename option) list -> t -> t
+
+  (** List of toplevel files *)
+  val with_toplevel : (basename optional * basename option) list -> t -> t
+
+  (** C bindings *)
+  val with_stublibs : (basename optional * basename option) list -> t -> t
+
+  (** List of architecture-independent files *)
+  val with_share : (basename optional * basename option) list -> t -> t
+
+  (** List of files under the more general share prefix *)
+  val with_share_root : (basename optional * basename option) list -> t -> t
+
+  (** List of etc files *)
+  val with_etc : (basename optional * basename option) list -> t -> t
+
+  (** List of doc files *)
+  val with_doc : (basename optional * basename option) list -> t -> t
+
+  (** Man pages *)
+  val with_man : (basename optional * basename option) list -> t -> t
+
+  (** Executable files under lib/ *)
+  val with_libexec : (basename optional * basename option) list -> t -> t
+
+  (** Not relative to the package's lib dir *)
+  val with_lib_root : (basename optional * basename option) list -> t -> t
+
+  (** Not relative to the package's lib dir, and with +x set *)
+  val with_libexec_root : (basename optional * basename option) list -> t -> t
+
+  (** List of other files to install *)
+  val with_misc : (basename optional * filename) list -> t -> t
+
 end
 
 (** .changes files, bound to the OpamDirTrack module *)
@@ -954,10 +1042,13 @@ module Switch_config: sig
     variables: (variable * variable_contents) list;
     opam_root: dirname option;
     wrappers: Wrappers.t;
-    env: env_update list;
+    env: (spf_resolved, euok_writeable) env_update list;
     invariant: OpamFormula.t option;
     depext_bypass: OpamSysPkg.Set.t;
   }
+
+  val env: t -> (spf_resolved, [> euok_writeable ]) env_update list
+
   val file_format_version: OpamVersion.t
   val variable: t -> variable -> variable_contents option
   val path: t -> std_path -> string option
