@@ -37,7 +37,7 @@ module VCS = struct
     | None -> mark_prefix
     | Some fragment -> mark_prefix ^ "-" ^ fragment
 
-  let fetch ?cache_dir:_ ?subpath:_ repo_root repo_url =
+  let fetch ?full_fetch:_ ?cache_dir:_ ?subpath:_ repo_root repo_url =
     let src = OpamUrl.base_url repo_url in
     let rev = OpamStd.Option.default "default" repo_url.OpamUrl.hash in
     let mark = mark_from_url repo_url in
@@ -84,7 +84,7 @@ module VCS = struct
     else
       Done (Some (OpamFilename.of_string patch_file))
 
-  let is_up_to_date repo_root repo_url =
+  let is_up_to_date ?subpath:_ repo_root repo_url =
     let mark = mark_from_url repo_url in
     hg repo_root [ "status"; "--subrepos"; "--rev"; mark ] @@> fun r ->
     OpamSystem.raise_on_process_error r;
