@@ -38,6 +38,19 @@ let load_config lock_kind global_lock root =
   let config =
     OpamFormatUpgrade.as_necessary lock_kind global_lock root config
   in
+  (* Update Cygwin variants cygbin *)
+  let cygbin =
+    let config = fst config in
+    let cygwin = OpamSysInteract.Cygwin.cygbin_opt config in
+    let cygbin =
+      if cygwin = None then
+        OpamSysInteract.Cygwin.msys2bin_opt config
+      else
+        cygwin
+    in
+    Option.map OpamFilename.Dir.to_string cygbin
+  in
+  OpamCoreConfig.update ?cygbin ();
   config
 
 let inferred_from_system = "Inferred from system"
@@ -60,7 +73,7 @@ let load lock_kind =
     OpamConsole.error_and_exit `Configuration_error
       "Opam has not been initialised, please run `opam init'";
   let config_lock = OpamFilename.flock lock_kind (OpamPath.config_lock root) in
-  let config =
+  let config, global_state_to_upgrade =
     try load_config lock_kind global_lock root
     with OpamFormatUpgrade.Upgrade_done _ as e ->
       OpamSystem.funlock config_lock;
@@ -97,7 +110,13 @@ let load lock_kind =
   in
   let eval_variables = OpamFile.Config.eval_variables config in
   let global_variables =
-    let env = lazy (OpamEnv.get_pure () |> OpamTypesBase.env_array) in
+    let env = lazy (
+      let env = OpamEnv.get_pure () |> OpamTypesBase.env_array in
+      match OpamCoreConfig.(!r.cygbin) with
+      | Some cygbin ->
+        OpamStd.Env.cyg_env ~env ~cygbin
+          ~git_location:OpamCoreConfig.(!r.git_location)
+      | None -> env) in
     List.fold_left (fun acc (v, cmd, doc) ->
         OpamVariable.Map.update v
           (fun previous_value ->
@@ -124,7 +143,9 @@ let load lock_kind =
   { global_lock = config_lock;
     root;
     config;
-    global_variables; }
+    global_variables;
+    global_state_to_upgrade;
+    }
 
 let switches gt =
   OpamFile.Config.installed_switches gt.config
@@ -212,4 +233,3 @@ let fix_switch_list gt =
       write gt, gt
     with OpamSystem.Locked -> gt
   else gt
-
