@@ -46,12 +46,12 @@ let package_variable_names = [
   "dev",       "True if this is a development package";
   "build-id",  "A hash identifying the precise package version with all its \
                 dependencies";
-  "opamfile", "Path of the curent opam file";
+  "opamfile", "Path of the current opam file";
 ]
 
 let predefined_depends_variables =
   List.map OpamVariable.Full.of_string [
-    "build"; "post"; "with-test"; "with-doc"; "dev";
+    "build"; "post"; "with-test"; "with-doc"; "with-dev-setup"; "dev";
   ]
 
 let resolve_global gt full_var =
@@ -134,6 +134,7 @@ let filter_depends_formula
     ?(post=false)
     ?(test=OpamStateConfig.(!r.build_test))
     ?(doc=OpamStateConfig.(!r.build_doc))
+    ?(dev_setup=OpamStateConfig.(!r.dev_setup))
     ?(dev=false)
     ?default
     ~env
@@ -143,9 +144,9 @@ let filter_depends_formula
   OpamFilter.partial_filter_formula (fun v ->
       if List.mem v predefined_depends_variables then None
       else env v) |>
-  OpamFilter.filter_deps ~build ~post ~test ~doc ~dev ?default
+  OpamFilter.filter_deps ~build ~post ~test ~doc ~dev_setup ~dev ?default
 
-let all_depends ?build ?post ?test ?doc ?dev ?(filter_default=false)
+let all_depends ?build ?post ?test ?doc ?dev_setup ?dev ?(filter_default=false)
     ?(depopts=true) st opam =
   let dev = match dev with None -> is_dev_package st opam | Some d -> d in
   let deps =
@@ -153,7 +154,7 @@ let all_depends ?build ?post ?test ?doc ?dev ?(filter_default=false)
       (OpamFile.OPAM.depends opam ::
        if depopts then [OpamFile.OPAM.depopts opam] else [])
   in
-  filter_depends_formula ?build ?post ?test ?doc ~dev
+  filter_depends_formula ?build ?post ?test ?doc ?dev_setup ~dev
     ~default:filter_default
     ~env:(resolve_switch ~package:(OpamFile.OPAM.package opam) st) deps
 
@@ -301,7 +302,7 @@ let resolve st ?opam:opam_arg ?(local=OpamVariable.Map.empty) v =
       OpamStd.Option.Op.(
         OpamFile.OPAM.url opam
         >>| OpamFile.URL.checksum
-        (* on download, the cache is populated with the first checksum found *)
+        >>| OpamHash.sort
         >>= (function [] -> None
                     | h::_ -> Some (string (OpamHash.to_string h))))
     | "dev", Some opam -> Some (bool (is_dev_package st opam))
