@@ -12,6 +12,7 @@
 open OpamTypes
 open OpamStateTypes
 open OpamFilename.Op
+open OpamStd.Option.Op
 
 let log fmt = OpamConsole.log "PIN" fmt
 
@@ -21,6 +22,8 @@ let package_opt st name = try Some (package st name) with Not_found -> None
 
 let version st name = (package st name).version
 
+let version_opt st name = try Some (version st name) with Not_found -> None
+
 let packages st = st.pinned
 
 let possible_definition_filenames dir name = [
@@ -32,27 +35,24 @@ let possible_definition_filenames dir name = [
   dir // "opam"
 ]
 
-let check_locked ?subpath default =
-  (* we keep the check, but this function shouldn't be called if the package is
-     not asked as locked *)
-  match OpamStateConfig.(!r.locked) with
-  | None -> default
-  | Some ext ->
-    let flo =
-      match subpath with
-      | Some s -> OpamFilename.(Op.(Dir.of_string s // to_string default))
-      | None -> default
-    in
-    let fl = OpamFilename.add_extension flo ext in
-    if not (OpamFilename.exists fl) then default else
-      (log "Lock file found %s" (OpamFilename.to_string flo);
+let lock_filename ?locked file =
+  locked
+  >>+ (fun () ->  OpamStateConfig.(!r.locked))
+  >>| (fun e -> OpamFilename.add_extension file e, e)
+
+let check_locked ?locked default =
+  match lock_filename ?locked default with
+  | None -> default, None
+  | Some (locked, ext) ->
+    if not (OpamFilename.exists locked) then default, None else
+      (log "Lock file found %s" (OpamFilename.to_string default);
        let base_depends =
-         OpamFile.make flo
+         OpamFile.make default
          |> OpamFile.OPAM.read
          |> OpamFile.OPAM.depends
        in
        let lock_depends =
-         OpamFile.make fl
+         OpamFile.make locked
          |> OpamFile.OPAM.read
          |> OpamFile.OPAM.depends
        in
@@ -63,11 +63,11 @@ let check_locked ?subpath default =
        in
        let base_formula =
          OpamFilter.filter_deps ~build:true ~post:true ~test:false ~doc:false
-           ~dev:false base_depends
+           ~dev_setup:false ~dev:false base_depends
        in
        let lock_formula =
          OpamFilter.filter_deps ~build:true ~post:true ~test:false ~doc:false
-           ~dev:false lock_depends
+           ~dev_setup:false ~dev:false lock_depends
        in
        let lpkg_f =
          lock_formula
@@ -118,7 +118,7 @@ let check_locked ?subpath default =
        let contains, consistent = fold base_formula in
        if contains <> [] || consistent <> [] then
          (OpamConsole.warning "Lock file %s is outdated, you may want to re-run opam lock:\n%s"
-            (OpamConsole.colorise `underline (OpamFilename.Base.to_string (OpamFilename.basename fl)))
+            (OpamConsole.colorise `underline (OpamFilename.Base.to_string (OpamFilename.basename locked)))
             ((if contains <> [] then
                 Printf.sprintf "Dependencies present in opam file not in lock file:\n%s"
                   (OpamStd.Format.itemize OpamPackage.Name.to_string contains)
@@ -137,21 +137,19 @@ let check_locked ?subpath default =
                             bv))
                       consistent)
               else "")));
-       OpamFilename.add_extension default ext)
+       locked, Some ext)
 
-let find_opam_file_in_source ?(locked=false) name dir =
+let find_opam_file_in_source ?locked name dir =
   let opt =
     OpamStd.List.find_opt OpamFilename.exists
       (possible_definition_filenames dir name)
   in
-  (match opt with
-   | Some base when locked -> Some (check_locked base)
-   | _ -> opt)
-  |> OpamStd.Option.map OpamFile.make
+  opt
+  >>| check_locked ?locked
+  >>| (fun (o,l) -> OpamFile.make o, l)
 
-let name_of_opam_filename dir file =
-  let open OpamStd.Option.Op in
-  let suffix = ".opam" in
+let name_of_opam_filename ?locked dir file =
+  let suffix= ".opam" in
   let get_name s =
     if Filename.check_suffix s suffix
     then Some Filename.(chop_suffix (basename s) suffix)
@@ -159,7 +157,7 @@ let name_of_opam_filename dir file =
   in
   let rel = OpamFilename.remove_prefix dir file in
   let rel =
-    match OpamStateConfig.(!r.locked) with
+    match locked with
     | None -> rel
     | Some suf ->
       let ext = "."^suf in
@@ -172,7 +170,7 @@ let name_of_opam_filename dir file =
   try Some (OpamPackage.Name.of_string name)
   with Failure _ -> None
 
-let files_in_source ?(recurse=false) ?subpath d =
+let files_in_source ?locked ?(recurse=false) ?subpath d =
   let baseopam = OpamFilename.Base.of_string "opam" in
   let files =
     let rec files_aux acc base d =
@@ -182,8 +180,10 @@ let files_in_source ?(recurse=false) ?subpath d =
                OpamFilename.check_suffix f ".opam" then
               let base =
                 match base, subpath with
-                | Some b, Some sp -> Some (Filename.concat sp b)
-                | Some b, _ | _, Some b -> Some b
+                | Some b, Some sp ->
+                  Some (Filename.concat (OpamFilename.SubPath.to_string sp) b)
+                | Some b, _ -> Some b
+                | _, Some sp -> Some (OpamFilename.SubPath.to_string sp)
                 | None, None -> None
               in
               Some (f, base)
@@ -218,29 +218,79 @@ let files_in_source ?(recurse=false) ?subpath d =
     in
     files_aux [] None
   in
-  let d =
-    (OpamStd.Option.map_default (fun sp -> OpamFilename.Op.(d / sp)) d subpath)
-  in
+  let d = OpamFilename.SubPath.(d /? subpath) in
   files d @ files (d / "opam") |>
-  List.map (fun (f,s) -> (check_locked ?subpath:s f), s) |>
+  List.map (fun (f,s) -> (check_locked ?locked f), s) |>
   OpamStd.List.filter_map
-    (fun (f, subpath) ->
+    (fun ((f, locked), subpath) ->
        try
          (* Ignore empty files *)
          if (Unix.stat (OpamFilename.to_string f)).Unix.st_size = 0 then None
-         else Some (name_of_opam_filename d f, OpamFile.make f, subpath)
+         else
+           Some { pin_name = name_of_opam_filename ?locked d f;
+                  pin = {
+                    pin_file = OpamFile.make f;
+                    pin_locked = locked;
+                    pin_subpath =
+                      OpamStd.Option.map OpamFilename.SubPath.of_string subpath;
+                    pin_url = ();
+                  }}
        with Unix.Unix_error _ ->
          OpamConsole.error "Can not read %s, ignored."
            (OpamFilename.to_string f);
          None)
 
+let files_in_source_w_target ?locked ?recurse ?subpath
+    ?(same_kind=fun _ -> true) url dir =
+  OpamStd.List.filter_map (fun name_and_file ->
+      let pin_url =
+        match url.OpamUrl.backend with
+        | #OpamUrl.version_control as vc ->
+          let module VCS =
+            (val match vc with
+               | `git -> (module OpamGit.VCS: OpamVCS.VCS)
+               | `hg -> (module OpamHg.VCS: OpamVCS.VCS)
+               | `darcs -> (module OpamDarcs.VCS: OpamVCS.VCS)
+               : OpamVCS.VCS)
+          in
+          let open OpamProcess.Job.Op in
+          let versioned_files =
+            OpamProcess.Job.run @@
+            VCS.versioned_files dir @@| fun files -> files
+          in
+          let opamfile =
+            OpamFilename.remove_prefix dir
+              (OpamFile.filename name_and_file.pin.pin_file)
+          in
+          if List.mem opamfile versioned_files
+          || not (OpamStd.String.contains opamfile ~sub:Filename.dir_sep) then
+            url
+          else
+            { url with
+              transport = "file";
+              hash = None;
+              backend = `rsync }
+        | _ -> url
+      in
+      if same_kind pin_url then
+        Some { name_and_file with pin = { name_and_file.pin with pin_url }}
+      else None)
+    (files_in_source ?locked ?recurse ?subpath dir)
+
 let orig_opam_file st name opam =
-  let open OpamStd.Option.Op in
   OpamFile.OPAM.get_metadata_dir
     ~repos_roots:(OpamRepositoryState.get_root st.switch_repos)
     opam >>= fun dir ->
-  OpamStd.List.find_opt OpamFilename.exists [
+  let opam_files = [
     dir // (OpamPackage.Name.to_string name ^ ".opam");
     dir // "opam"
-  ] >>|
-  OpamFile.make
+  ] in
+  let locked_files =
+    match OpamFile.OPAM.locked opam with
+    | Some locked ->
+      List.map (fun f -> OpamFilename.add_extension f locked) opam_files
+    | None -> []
+  in
+  OpamStd.List.find_opt OpamFilename.exists locked_files
+  ++ OpamStd.List.find_opt OpamFilename.exists opam_files
+  >>| OpamFile.make
