@@ -78,11 +78,14 @@ module Cache = struct
 end
 
 let load_opams_from_dir repo_name repo_root =
+  if OpamConsole.disp_status_line () || OpamConsole.verbose () then
+    OpamConsole.status_line "Processing: [%s: loading data]"
+      (OpamConsole.colorise `blue (OpamRepositoryName.to_string repo_name));
   (* FIXME: why is this different from OpamPackage.list ? *)
   let rec aux r dir =
     if OpamFilename.exists_dir dir then
       let fnames = Sys.readdir (OpamFilename.Dir.to_string dir) in
-      if Array.fold_left (fun a f -> a || f = "opam") false fnames then
+      if Array.exists (fun f -> f = "opam") fnames then
         match OpamFileTools.read_repo_opam ~repo_name ~repo_root dir with
         | Some opam ->
           (try
@@ -104,7 +107,9 @@ let load_opams_from_dir repo_name repo_root =
           r fnames
     else r
   in
-  aux OpamPackage.Map.empty (OpamRepositoryPath.packages_dir repo_root)
+  Fun.protect
+    (fun () -> aux OpamPackage.Map.empty (OpamRepositoryPath.packages_dir repo_root))
+    ~finally:OpamConsole.clear_status
 
 let load_repo repo repo_root =
   let t = OpamConsole.timer () in
@@ -150,6 +155,7 @@ let get_repo_root rt repo =
   get_root_raw rt.repos_global.root rt.repos_tmp repo.repo_name
 
 let load lock_kind gt =
+  OpamFormatUpgrade.as_necessary_repo_switch_light_upgrade lock_kind `Repo gt;
   log "LOAD-REPOSITORY-STATE %@ %a" (slog OpamFilename.Dir.to_string) gt.root;
   let lock = OpamFilename.flock lock_kind (OpamPath.repos_lock gt.root) in
   let repos_map = OpamStateConfig.Repos.safe_read ~lock_kind gt in
