@@ -9,54 +9,9 @@
 (*                                                                        *)
 (**************************************************************************)
 
-(* Simple CRAM-like test framework for opam tests.
-   Features and format:
-   - first line is
-     * the git hash of the opam repository to use, an opamroot is already
-       initialised with that repo as "default"
-     * or N0REP0 for no dependency on opam repository, and an opamroot is
-       already initialised with an empty `default` repository in `./REPO`
-       directory, that you need to populate
-   - 'opam' is automatically redirected to the correct binary
-   - the command prefix is `### `
-   - use `### <FILENAME>`, then the contents below to create a file verbatim
-   - use `### <pkg:NAME.VERSION>`, then the contents of an opam file below to
-     add this package to `default` repository in `./REPO`
-   - use `### <pkg:NAME.VERSION:FILENAME>`, then the contents below to add this
-     file as a extra-file of the given package in the `default` repository, and
-     implicitely run `opam update default`
-   - use `### <pin:path>`, then the contents below to create a minimal opam
-     file, it is extended by template defined fields to pin it without lint
-     errors
-   - `### FOO=x BAR=y` to export variables for subsequent commands
-   - shell-like command handling:
-     * **NO pattern expansion, shell pipes, sequences or redirections**
-     * `FOO=x BAR=y command`
-     * Arguments can be quoted: eg `"foo\"bar"`, `'foo\bar'`, but not combined
-       (`foo'bar'` is not translated to `foobar`)
-     * Variable expansion in arguments (`$FOO` or `${FOO}`). Undefined variables
-       are left as-is
-     * rewrites:
-       * `| 'REGEXP' -> 'STR'` (can be repeated; set `STR` to `\c` to
-         clear the line)
-       * `| grep REGEXP`
-       * `| grep -v REGEXP`
-       * `| unordered` compares lines without considering their ordering
-       * `| sed-cmd command` replaces full path resolved command by `command`
-     * variables from command outputs: `cmd args >$ VAR`
-     * `### : comment`
-     * `opam-cat file`: prints a normalised opam file
-     * `json-cat file`: print a human readable opam output json file, with
-       replacement of some duration and temporary files names
-   - if you need more shell power, create a script using <FILENAME> then run it.
-     Or just use `sh -c`... but beware for compatibility.
-
-   The opam roots are generated using dynamically generated dune rules (see
-   gen.ml and dune.inc), then the tests are run using this script.
-*)
-
 type test = {
   repo_hash: string;
+  tags : string list;
   commands: (string * string list) list;
 }
 
@@ -92,8 +47,10 @@ v}*)
 
 let load_test f =
   let ic = open_in f in
-  let repo_hash = try input_line ic with
-    | End_of_file -> failwith "Malformed test file"
+  let repo_hash, tags =
+    match String.split_on_char ' ' (input_line ic) with
+    | [] | exception End_of_file -> failwith "Malformed test file"
+    | h::t -> h, t
   in
   let commands =
     let rec aux commands =
@@ -109,7 +66,7 @@ let load_test f =
     aux []
   in
   close_in ic;
-  { repo_hash; commands }
+  { repo_hash; tags; commands }
 
 let base_env =
   let propagate v = try [v, Sys.getenv v] with Not_found -> [] in
@@ -303,18 +260,27 @@ let rec with_temp_dir f =
   (mkdir_p s;
    finally f s @@ fun () -> rm_rf s)
 
+type filter = (Re.t * filt_sort) list
+
 type command =
   | File_contents of string
-  | Repo_pkg_file_contents of string
+  | Repo_pkg_file_contents of
+      string * string (* name * version *)
+      * [ `opam
+        | `files of string (* file name *) ]
   | Pin_file_content of string
-  | Cat of { files: string list;
-             filter: (Re.t * filt_sort) list; }
+  | Opamfile of { files: string list;
+             filter: filter; }
   | Json of { files: string list;
-             filter: (Re.t * filt_sort) list; }
+             filter: filter; }
+  | Cache of { kind: [`installed | `repo];
+               switch: string;
+               nvs: string list;
+               filter: filter; }
   | Run of { env: (string * string) list;
              cmd: string;
              args: string list; (* still escaped *)
-             filter: (Re.t * filt_sort) list;
+             filter: filter;
              output: string option;
              unordered: bool;
              sort: bool;}
@@ -367,7 +333,7 @@ module Parse = struct
       group @@ seq [ alpha; rep @@ alt [ alnum; set "_-+" ]];
       char '.';
       group @@ rep1 @@ alt [ alnum; set "-_+.~" ];
-      opt @@ seq [ char ':' ; group @@ rep1 @@ alt [ alnum; set "-_+.~" ]];
+      opt @@ seq [ char ':' ; group @@ rep1 @@ alt [ alnum; set "-_+.~" ] ];
       char '>'
     ]
 
@@ -376,26 +342,24 @@ module Parse = struct
       if String.length str > 4 && String.sub str 1 4 = "pin:" then
         Pin_file_content (String.sub str 5 (String.length str - 6))
       else
-      try
-        let grs = exec (compile re_package) str in
-        let name = Group.get grs 1 in
-        let version = Group.get grs 2 in
-        Repo_pkg_file_contents
-          (try
-             let files = Group.get grs 3 in
-             Printf.sprintf "%s/packages/%s/%s.%s/files/%s"
-               default_repo name name version files
-           with Not_found ->
-             Printf.sprintf "%s/packages/%s/%s.%s/opam"
-               default_repo name name version)
-      with Not_found ->
-        File_contents (String.sub str 1 (String.length str - 2))
-    else if str.[0] = ':' || str.[0] = '#' then
-      Comment str
+        try
+          let grs = exec (compile re_package) str in
+          let name = Group.get grs 1 in
+          let version = Group.get grs 2 in
+          Repo_pkg_file_contents
+            (name, version,
+             try
+               let file = Group.get grs 3 in
+               `files (file)
+             with Not_found -> `opam)
+        with Not_found ->
+          File_contents (String.sub str 1 (String.length str - 2))
+      else if str.[0] = ':' || str.[0] = '#' then
+        Comment str
     else
-    let varbinds, pos =
-      let gr = exec (compile @@ rep re_varbind) str in
-      List.map (fun gr ->
+      let varbinds, pos =
+        let gr = exec (compile @@ rep re_varbind) str in
+        List.map (fun gr ->
           Group.get gr 1,
           (match Group.get gr 2 with
            | "=" -> `eq
@@ -405,18 +369,18 @@ module Parse = struct
           get_str (Group.get gr 3))
         (all (compile @@ re_varbind) (Group.get gr 0)),
       Group.stop gr 0
-    in
+        in
     let cmd, pos =
       try
         let gr = exec ~pos (compile re_str_atom) str in
         Some (get_str (Group.get gr 0)),
         Group.stop gr 0
-      with Not_found -> None, pos
-    in
+          with Not_found -> None, pos
+        in
     let args =
       let grs = all ~pos (compile re_str_atom) str in
       List.map (fun gr -> Group.get gr 0) grs
-    in
+      in
     let get_str ?escape s =
       str_replace_path ?escape OpamSystem.back_to_forward
         (filters_of_var vars)
@@ -430,29 +394,29 @@ module Parse = struct
     let rec get_args_rewr acc = function
       | [] -> List.rev acc, false, false, [], None
       | ("|"|">$") :: _ as rewr ->
-        let rec get_rewr (unordered, sort, acc) = function
-          | "|" :: re :: "->" :: str :: r ->
-            get_rewr (unordered, sort, (posix_re re, Sed (get_str str)) :: acc) r
-          | "|" :: "grep" :: "-v" :: re :: r ->
-            get_rewr (unordered, sort, (posix_re re, GrepV) :: acc) r
-          | "|" :: "grep" :: re :: r ->
-            get_rewr (unordered, sort, (posix_re re, Grep) :: acc) r
-          | "|" :: "sort" :: r ->
-            if acc <> [] then
-              Printf.printf "Warning: sort should appear _before_ any filters\n%!";
+          let rec get_rewr (unordered, sort, acc) = function
+            | "|" :: re :: "->" :: str :: r ->
+                get_rewr (unordered, sort, (posix_re re, Sed (get_str str)) :: acc) r
+            | "|" :: "grep" :: "-v" :: re :: r ->
+                get_rewr (unordered, sort, (posix_re re, GrepV) :: acc) r
+            | "|" :: "grep" :: re :: r ->
+                get_rewr (unordered, sort, (posix_re re, Grep) :: acc) r
+            | "|" :: "sort" :: r ->
+                if acc <> [] then
+                  Printf.printf "Warning: sort should appear _before_ any filters\n%!";
             get_rewr (unordered, true, acc) r
-          | "|" :: "unordered" :: r ->
-            get_rewr (true, sort, acc) r
-          | "|" :: "sed-cmd" :: cmd :: r ->
-            let sandbox =
-              (* Sandbox prefix
+            | "|" :: "unordered" :: r ->
+                get_rewr (true, sort, acc) r
+            | "|" :: "sed-cmd" :: cmd :: r ->
+                let sandbox =
+                  (* Sandbox prefix
                  >[...] /tmp/build_592d92_dune/opam-reftest-2b89f9/OPAM/opam-init/hooks/sandbox.sh "build" "cmd" <
                  >[...] ${BASEDIR}/opam-init/hooks/sandbox.sh "build" "cmd" <
                  -->
-                 >[...] cmd <
+                   >[...] cmd <
               *)
-              seq [
-                alt [ char '/'; Re.str "${" ];
+                  seq [
+                    alt [ char '/'; Re.str "${" ];
                 non_greedy @@ rep1 any; Re.str "sandbox.sh";
                 space;
                 char '"';
@@ -461,25 +425,25 @@ module Parse = struct
                 space;
                 char '"'; Re.str cmd; char '"';
                 space;
-              ]
-            in
+                ]
+    in
             let with_quote_set s = set ("\"'"^s) in
             let opt_quoted r = [
               seq @@ [ char '"'] @  r @ [ char '"'; rep1 space ];
               seq @@ r @ [ rep1 space ];
-            ] in
+              ] in
             let unix_prefix =
               (* Unix & Mac command prefix
                  >[...] /usr/bin/cmd <
                  >[...] /usr/bin/cmd <
                  -->
-                 >[...] cmd <
+                   >[...] cmd <
               *)
               opt_quoted @@ [
                 rep1 @@ seq [ char '/'; rep1 @@ diff any (with_quote_set "/ ") ];
                 char '/';
                 Re.str cmd;
-              ]
+                ]
             in
             let win_prefix =
               (* Windows command prefix
@@ -487,7 +451,7 @@ module Parse = struct
                  >[...] C:\Windows\system32\cmd <
                  >[...] C:\Windows\system 32\cmd <
                  -->
-                 >[...] cmd <
+                   >[...] cmd <
               *)
               opt_quoted @@ [
                 alpha; char ':';
@@ -496,43 +460,58 @@ module Parse = struct
                 char '\\'; opt @@ char '\\';
                 Re.str cmd;
                 opt @@ Re.str ".exe";
-              ] in
+                ] in
             let re = alt @@ sandbox :: unix_prefix @ win_prefix in
             let str = Printf.sprintf "%s " cmd in
             get_rewr (unordered, sort, (re, Sed str) :: acc) r
-          | ">$" :: output :: [] ->
-            unordered, sort, List.rev acc, Some (get_str output)
-          | [] ->
-            unordered, sort, List.rev acc, None
-          | r ->
-            Printf.printf
+            | ">$" :: output :: [] ->
+                unordered, sort, List.rev acc, Some (get_str output)
+            | [] ->
+                unordered, sort, List.rev acc, None
+            | r ->
+                Printf.printf
               "Bad rewrite %S, expecting '| RE -> STR' or '>$ VAR'\n%!"
               (String.concat " " r);
             unordered, sort, List.rev acc, None
-        in
+            in
         let unordered, sort, rewr, out = get_rewr (false, false, []) rewr in
         List.rev acc, unordered, sort, rewr, out
-      | arg :: r -> get_args_rewr (arg :: acc) r
-    in
+            | arg :: r -> get_args_rewr (arg :: acc) r
+        in
     let args, unordered, sort, rewr, output = get_args_rewr [] args in
     match cmd with
     | Some "opam-cat" ->
-      Cat { files = args; filter = rewr; }
+        Opamfile { files = args; filter = rewr; }
     | Some "json-cat" ->
-      Json { files = args; filter = rewr; }
+        Json { files = args; filter = rewr; }
+    | Some "opam-cache" ->
+      let kind, switch, nvs =
+        match args with
+        | "installed"::switch::nvs ->
+          `installed, switch, nvs
+        | "repo"::nvs ->
+          `repo, "", nvs
+        | _ ->
+          failwith
+            (Printf.sprintf
+               "Bad usage of opam-cache %s.\n\
+                expecting 'opam-cache <installed|repo> <switch?> [nvs]"
+             (String.concat " " args))
+      in
+      Cache { kind; switch; nvs; filter = rewr; }
     | Some cmd ->
-      let env, plus =
-        List.fold_left (fun (env,plus) (v,op,value) ->
+        let env, plus =
+          List.fold_left (fun (env,plus) (v,op,value) ->
             match op with
             | `eq -> (v,value)::env, plus
             | `pluseq -> env, (v^"+="^value)::plus
             | `eqplus -> env, (v^"=+"^value)::plus)
           ([],[]) varbinds
-      in
+    in
       (match plus with
        | [] -> ()
        | _ ->
-         OpamConsole.error
+           OpamConsole.error
            "variable bindings at the beginning of a command does not \
             support '+=' or '=+' operators: %s"
            (OpamStd.Format.pretty_list plus));
@@ -544,10 +523,10 @@ module Parse = struct
         output;
         unordered;
         sort;
-      }
-    | None ->
-      Export varbinds
-end
+        }
+       | None ->
+           Export varbinds
+    end
 
 let parse_command = Parse.command
 
@@ -679,14 +658,18 @@ dev-repo: "hg+https://pkg@op.am"
 bug-reports: "https://nobug"
 |} "<nofile>"
 
-let print_file ~filters reader names =
+let print_file ~filters ?(single_header=true) names_and_content =
   let files =
-    match names with
-    | [file] -> [reader file]
-    | files  ->
+    let with_name () =
       List.map
-        (fun f -> Printf.sprintf "=> %s <=\n%s" f (reader f))
-        files
+        (fun (name, content) ->
+           Printf.sprintf "=> %s <=\n%s" name content)
+        names_and_content
+    in
+    if single_header then with_name () else
+      match names_and_content with
+      | [_, content] -> [content]
+      | _ -> with_name ()
   in
   List.map (fun s -> OpamStd.String.split s '\n') files
   |> List.flatten
@@ -697,6 +680,86 @@ let print_file ~filters reader names =
       in
       if not (String.equal s "\\c") then print_string (s^"\n"))
 
+let add_extra_files fpath opamfile file =
+  let replace f ~default l =
+    let found, nl =
+      List.fold_left (fun (found, nl) item ->
+          if found then found, item::nl else
+            match f item with
+            | Some y -> true, y::nl
+            | None -> false, item::nl)
+        (false, []) l
+    in
+    (if not found then default::nl else nl)
+    |> List.rev
+  in
+  let open OpamParserTypes.FullPos in
+  let pos = (* null pos *)
+    { filename = opamfile.file_name; start = -1, -1; stop = -1, -1; }
+  in
+  let md5 = OpamHash.(to_string (compute ~kind:`MD5 fpath)) in
+  let xf_pair =
+    [{ pelem = String file; pos };
+     { pelem = String md5; pos }]
+  in
+  let xf_pair_list =
+    { pelem = List { pelem = xf_pair; pos}; pos}
+  in
+  let default =
+    { pelem = Variable ({ pelem = "extra-files"; pos},
+                        xf_pair_list); pos}
+  in
+  let with_field xfiles =
+    Some { pelem = Variable
+               ({ pelem = "extra-files"; pos},
+                { pelem = List { pelem = xfiles ; pos}; pos}); pos}
+  in
+  let file_contents =
+    replace ~default
+      (function
+        | { pelem = Variable
+                ({ pelem = "extra-files"; _},
+                 { pelem = List { pelem = xfiles ; _}; _}); _} ->
+          (match xfiles with
+           | [{ pelem = String xf; _};
+              { pelem = String _checksum; _}] as xfield ->
+             with_field @@
+             if String.equal xf file then
+               xf_pair
+             else
+               [{ pelem = List { pelem = xfield; pos}; pos};
+                { pelem = List { pelem = xf_pair; pos}; pos}]
+           | { pelem = List _; _}::_ ->
+             (* There is only 2 construction
+                * 1 file, list of 2 strings
+                  * extra-files: [ "file" "checksum" ]
+                * list of files
+                  * extra-files: [[ "file" "checksum" ]]
+                  * extra-files: [[ "file" "checksum" ] [ "file2" "checkums2"]]
+             *)
+             assert (List.for_all (function
+                   { pelem = List
+                         { pelem = [{ pelem = String _; _ };
+                                    { pelem = String _; _}];
+                           _}; _} -> true
+                 | _ -> false) xfiles);
+             with_field @@
+             replace (function
+                 | { pelem = List
+                         { pelem =
+                             [ { pelem = String xf; _ };
+                               { pelem = String _; _}]; _}; _}
+                   when String.equal xf file ->
+                   Some xf_pair_list
+                 | _ -> None)
+               ~default:xf_pair_list xfiles
+           (* cf comment above *)
+           | _ -> assert false)
+        | _ -> None)
+      opamfile.file_contents
+  in
+  { opamfile with file_contents }
+
 let run_test ?(vars=[]) ~opam t =
   let old_cwd = Sys.getcwd () in
   let opamroot0 = Filename.concat old_cwd ("root-"^t.repo_hash) in
@@ -726,7 +789,7 @@ let run_test ?(vars=[]) ~opam t =
   ignore @@ command ~silent:true opam.as_called
     ["var"; "--quiet"; "--root"; opamroot; "--global"; "--cli=2.1";
      "sys-ocaml-version=4.08.0"];
-  print_endline t.repo_hash;
+  print_endline (String.concat " " (t.repo_hash::t.tags));
   let _vars =
     List.fold_left (fun vars (cmd, out) ->
         print_string cmd_prompt;
@@ -738,9 +801,53 @@ let run_test ?(vars=[]) ~opam t =
           write_file ~path ~contents;
           print_string contents;
           vars
-        | Repo_pkg_file_contents path ->
+        | Repo_pkg_file_contents (name, version, repo_file) ->
           let contents = String.concat "\n" out ^ "\n" in
-          write_file ~path ~contents;
+          let opam_path =
+            Printf.sprintf "%s/packages/%s/%s.%s/opam"
+              default_repo name name version
+          in
+          let file_path file =
+            Printf.sprintf "%s/packages/%s/%s.%s/files/%s"
+              default_repo name name version file
+          in
+          (match repo_file with
+           | `opam ->
+             let contents =
+               try
+                 (* update with extra files if found *)
+                 let files =
+                   Filename.concat (Filename.dirname opam_path) "files"
+                   |> Sys.readdir
+                   |> Array.to_list
+                   |> List.filter (fun f ->
+                       not (Sys.is_directory (file_path f)))
+                 in
+                 match files with
+                 | [] -> contents
+                 | _ ->
+                   let opamfile = OpamParser.FullPos.string contents opam_path in
+                   List.fold_left (fun opamfile file ->
+                       add_extra_files (file_path file) opamfile file)
+                     opamfile files
+                   |> OpamPrinter.FullPos.opamfile
+               with Sys_error _ -> contents
+             in
+             (* and write opam file *)
+             write_file ~path:opam_path ~contents;
+           | `files file ->
+             (* first update the given file *)
+             let fpath = file_path file in
+             write_file ~path:fpath ~contents;
+             (* then update the opam file *)
+             if Sys.file_exists opam_path then
+               let opamfile = OpamParser.FullPos.file opam_path in
+               let contents =
+                 add_extra_files fpath opamfile file
+                 |> OpamPrinter.FullPos.opamfile
+               in
+               write_file ~path:opam_path ~contents
+          );
           print_string contents;
           ignore @@ run_cmd ~opam ~dir ~vars ~silent:true
             "opam" ["update"; "default"];
@@ -804,13 +911,18 @@ let run_test ?(vars=[]) ~opam t =
                in
                (v, value) :: List.filter (fun (w, _) -> not (String.equal v w)) vars)
             vars bindings
-        | Cat { files; filter } ->
+        | Opamfile { files; filter } ->
           let files =
-            List.map (fun s -> Re.(replace_string (compile @@ str "$OPAMROOT")
-                                     ~by:opamroot s)) files
+            List.map (fun s ->
+                let name =
+                  Re.(replace_string (compile @@ str "$OPAMROOT")
+                        ~by:opamroot s)
+                in
+                name, print_opamfile name)
+              files
           in
-          print_file ~filters:(filter @ common_filters dir)
-            print_opamfile files;
+          let filters = filter @ common_filters dir in
+          print_file ~single_header:false ~filters files;
           vars
         | Json { files; filter } ->
           let files =
@@ -828,12 +940,16 @@ let run_test ?(vars=[]) ~opam t =
             in
             [
               seq [ str {|"duration": |};
-                    rep1 digit; char '.';
-                    rep1 @@ alt [digit ; char 'e'; char '-'] ],
+                    rep1 digit;
+                    opt @@ seq [ char '.';
+                                 rep1 @@ alt [digit ; char 'e'; char '-']];
+                    eow ],
               Sed {|"duration": 6.2831853071|};
               hex_output ".env";
               hex_output ".out";
               str OpamVersion.(to_string (full ())), Sed "currentv";
+              (* For opam Windows path *)
+              str (String.escaped opam.as_seen_in_opam), Sed "${OPAM}";
             ]
           in
           let to_string f =
@@ -853,8 +969,103 @@ let run_test ?(vars=[]) ~opam t =
               OpamJson.to_string ~minify:false json ^ "\n"
             | None -> "# Return Error reading json\n"^content
           in
-          print_file ~filters:(filter @ common_filters ~opam dir @ json_filters)
-            to_string files;
+          let files =
+            List.map (fun s ->
+                let name =
+                  Re.(replace_string (compile @@ str "$OPAMROOT")
+                        ~by:opamroot s)
+                in
+                name, to_string name)
+              files
+          in
+          let filters = filter @ common_filters ~opam dir @ json_filters in
+          print_file ~single_header:false ~filters files;
+          vars
+        | Cache { switch; kind; nvs; filter } ->
+          let nvs =
+            List.fold_left (fun nvs s ->
+                match OpamPackage.of_string_opt s with
+                | Some nv ->
+                  let n = OpamPackage.name nv in
+                  let v = OpamPackage.version nv in
+                  OpamPackage.Name.Map.update n
+                    (OpamPackage.Version.Set.add v)
+                    (OpamPackage.Version.Set.singleton v)
+                    nvs
+                | None ->
+                  OpamPackage.Name.Map.add
+                    (OpamPackage.Name.of_string s)
+                    OpamPackage.Version.Set.empty
+                    nvs)
+              OpamPackage.Name.Map.empty nvs
+          in
+          (match kind with
+           | `installed ->
+             (let cache =
+                OpamSwitchState.Installed_cache.load
+                  (OpamPath.Switch.installed_opams_cache
+                     (OpamFilename.Dir.of_string opamroot)
+                     (OpamSwitch.of_string switch))
+              in
+              match cache with
+              | None -> print_string "No cache\n"
+              | Some cache when OpamPackage.Map.is_empty cache ->
+                print_string "Empty cache\n"
+              | Some cache ->
+                let cache =
+                  if OpamPackage.Name.Map.is_empty nvs then cache else
+                    OpamPackage.Map.filter (fun nv _ ->
+                        let n = OpamPackage.name nv in
+                        match OpamPackage.Name.Map.find_opt n nvs with
+                        | Some vs ->
+                          OpamPackage.Version.Set.is_empty vs
+                          || OpamPackage.Version.Set.mem
+                            (OpamPackage.version nv) vs
+                        | None -> false)
+                      cache
+                in
+                let files =
+                  OpamPackage.Map.fold (fun pkg opam files ->
+                      let name = OpamPackage.to_string pkg in
+                      let content = OpamFile.OPAM.write_to_string opam in
+                      (name, content)::files)
+                    cache []
+                in
+                print_file ~filters:(filter @ common_filters dir) files)
+           | `repo ->
+             (let cache =
+                OpamRepositoryState.Cache.load
+                  (OpamFilename.Dir.of_string opamroot)
+              in
+              match cache with
+              | None -> print_string "No cache\n"
+              | Some (_, cache) when OpamRepositoryName.Map.is_empty cache ->
+                print_string "Empty cache\n"
+              | Some (_, cache) ->
+                let cache =
+                  if OpamPackage.Name.Map.is_empty nvs then cache else
+                    OpamRepositoryName.Map.map
+                      (OpamPackage.Map.filter (fun nv _ ->
+                           let n = OpamPackage.name nv in
+                           match OpamPackage.Name.Map.find_opt n nvs with
+                           | Some vs ->
+                             OpamPackage.Version.Set.is_empty vs
+                             || OpamPackage.Version.Set.mem
+                               (OpamPackage.version nv) vs
+                           | None -> false))
+                      cache
+                in
+                let files =
+                  OpamRepositoryName.Map.fold (fun reponame pkgmap files->
+                      let pre = OpamRepositoryName.to_string reponame in
+                      OpamPackage.Map.fold (fun pkg opam files ->
+                          let name = pre ^ ":" ^ OpamPackage.to_string pkg in
+                          let content = OpamFile.OPAM.write_to_string opam in
+                          (name, content)::files)
+                        pkgmap files)
+                    cache []
+                in
+                print_file ~filters:(filter @ common_filters dir) files));
           vars
         | Run {env; cmd; args; filter; output; unordered; sort} ->
           let silent = output <> None || unordered in
